@@ -1,6 +1,6 @@
 package NCO_benchmarks;
 
-# $Header: /cvsroot/nco/nco/bm/NCO_benchmarks.pm,v 1.18 2006/11/23 19:47:22 zender Exp $
+# $Header: /cvsroot/nco/nco/bm/NCO_benchmarks.pm,v 1.19 2011/02/21 05:41:29 zender Exp $
 
 # Purpose: library module supporting nco_bm.pl benchmark and regression tests
 # File contains BENCHMARK code (as opposed to the REGRESSION tests in "NCO_rgr.pm")
@@ -102,7 +102,7 @@ sub benchmarks{
 # SS_gnarly_pything() sub from tst_run() that handles the SS manipulations.
 # Also note that in order for Daniel's ssdwrap code to work at least for now, all the files passed in have
 # to be named differently in order to keep things straight.  So we can't name everything $fl_out
-# (or %tempf_00% and have it work. All output must be named differently in a script for the
+# (or %tmp_fl_00% and have it work. All output must be named differently in a script for the
 #script to work correctly.
 
 if (0) { # DEBUGGING to skip these
@@ -116,9 +116,9 @@ if (0) { # DEBUGGING to skip these
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 # csz 20061024: Changed ncap to ncap2 with 40% speed penalty
 # Not sure why ncap2 is much slower at long arithmetic expressions
-		$tst_cmd[0] = "ncap2 -h -O $fl_fmt $nco_D_flg -s \"nu_var1[time,lev,lat,lon]=d4_01*d4_02*(d4_03^2)-(d4_05/d4_06)\" -s \"nu_var2[time,lev,lat,lon]=(d4_13/d4_02)*((d4_03^2)-(d4_05/d4_06))\" -s \"nu_var3[time,lat,lon]=(d3_08*d3_01)-(d3_05^3)-(d3_11*d3_16)\" -s \"nu_var4[time,lat,lon]=(d3_08+d3_01)-((d3_05*3)-d3_11-17.33)\" $in_pth_arg gcm_T85.nc %tempf_01%";
-		$tst_cmd[1] = "ncwa -O $omp_flg -y sqrt -a lat,lon %tempf_01% %tempf_02%";
-		$tst_cmd[2] = "ncks -C -H -s '%f' -v d2_00  %tempf_02% ";
+		$tst_cmd[0] = "ncap2 -h -O $fl_fmt $nco_D_flg -s \"nu_var1[time,lev,lat,lon]=d4_01*d4_02*(d4_03^2)-(d4_05/d4_06)\" -s \"nu_var2[time,lev,lat,lon]=(d4_13/d4_02)*((d4_03^2)-(d4_05/d4_06))\" -s \"nu_var3[time,lat,lon]=(d3_08*d3_01)-(d3_05^3)-(d3_11*d3_16)\" -s \"nu_var4[time,lat,lon]=(d3_08+d3_01)-((d3_05*3)-d3_11-17.33)\" $in_pth_arg gcm_T85.nc %tmp_fl_01%";
+		$tst_cmd[1] = "ncwa -O $omp_flg -y sqrt -a lat,lon %tmp_fl_01% %tmp_fl_02%";
+		$tst_cmd[2] = "ncks -C -H -s '%f' -v d2_00  %tmp_fl_02% ";
 # as noted above, for serverside prep, the final '%stdouterr%' il be added in SS_gnarly_pything()
 # note that the 2 additions to the list are the expected value and the indicator whether it
 # can be used as a server side operation:
@@ -139,11 +139,11 @@ if ($dbg_lvl >= 1) {print "paused after ncap2 - hit return to continue"; $wait =
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncbo -h -O $fl_fmt $nco_D_flg $omp_flg --op_typ='-' $in_pth_arg gcm_T85.nc gcm_T85_00.nc %tempf_00%";
-#		$tst_cmd[0] = "ncbo -h -O $fl_fmt $nco_D_flg $omp_flg --op_typ='-' $in_pth_arg stl_sml.nc stl_sml_00.nc %tempf_00%";  # smaller test file
+		$tst_cmd[0] = "ncbo -h -O $fl_fmt $nco_D_flg $omp_flg --op_typ='-' $in_pth_arg gcm_T85.nc gcm_T85_00.nc %tmp_fl_00%";
+#		$tst_cmd[0] = "ncbo -h -O $fl_fmt $nco_D_flg $omp_flg --op_typ='-' $in_pth_arg stl_sml.nc stl_sml_00.nc %tmp_fl_00%";  # smaller test file
 		if($dbg_lvl > 2){print "entire cmd: $tst_cmd[0]\n";}
-		$tst_cmd[1] = "ncks -C -H -s '%f' -v sleepy %tempf_00% ";
-#		$tst_cmd[1] = "ncks -C -H -s '%f' -v weepy %tempf_00%"; # smaller test file
+		$tst_cmd[1] = "ncks -C -H -s '%f' -v sleepy %tmp_fl_00% ";
+#		$tst_cmd[1] = "ncks -C -H -s '%f' -v weepy %tmp_fl_00%"; # smaller test file
 		$tst_cmd[2] = "0.000000";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -159,10 +159,10 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 		if ($dbg_lvl > 0) {print "\nBenchmark: \$fl_fmt = [$fl_fmt], \$nco_D_flg = $nco_D_flg, \$omp_flg = [$omp_flg], \$dsc_sng = $dsc_sng, \$fl_cnt = [$fl_cnt], \n";}
-		$tst_cmd[0] = "ncea -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg stl_5km_00.nc %tempf_00%";
+		$tst_cmd[0] = "ncea -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg stl_5km_00.nc %tmp_fl_00%";
 		if($dbg_lvl > 2){print "entire cmd: $tst_cmd[0]\n";}
-		$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y sqrt -a lat,lon %tempf_00% %tempf_01%";
-		$tst_cmd[2] = "ncks -C -H -s '%f' -v d2_00 %tempf_01%";
+		$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y sqrt -a lat,lon %tmp_fl_00% %tmp_fl_01%";
+		$tst_cmd[2] = "ncks -C -H -s '%f' -v d2_00 %tmp_fl_01%";
 		$tst_cmd[3] = "1.604304";
 		$tst_cmd[4] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -178,9 +178,9 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng, files=$fl_cnt\n";}
-		$tst_cmd[0] = "ncecat -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg tms_lng_00.nc %tempf_00%";
-		$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg %tempf_00% %tempf_01%";
-		$tst_cmd[2] = "ncks -C -H -s '%f' -v PO2 %tempf_01%";
+		$tst_cmd[0] = "ncecat -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg tms_lng_00.nc %tmp_fl_00%";
+		$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg %tmp_fl_00% %tmp_fl_01%";
+		$tst_cmd[2] = "ncks -C -H -s '%f' -v PO2 %tmp_fl_01%";
 		# following required due to shortened length of test under dap.
 		if ($dodap eq "FALSE") { $tst_cmd[3] = "12.759310";}
 		else                   { $tst_cmd[3] = "18.106375";}
@@ -198,10 +198,10 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncflint -h -O $fl_fmt $nco_D_flg   -w '0.5' $in_pth_arg gcm_T85_00.nc  gcm_T85_01.nc %tempf_00%";
+		$tst_cmd[0] = "ncflint -h -O $fl_fmt $nco_D_flg   -w '0.5' $in_pth_arg gcm_T85_00.nc  gcm_T85_01.nc %tmp_fl_00%";
 		if($dbg_lvl > 2){print "entire cmd: $tst_cmd[0]\n";}
-		$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y sqrt -a lat,lon %tempf_00% %tempf_01%";
-		$tst_cmd[2] = "ncks -C -H -s '%f ' -v d1_00 %tempf_01%";
+		$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y sqrt -a lat,lon %tmp_fl_00% %tmp_fl_01%";
+		$tst_cmd[2] = "ncks -C -H -s '%f ' -v d1_00 %tmp_fl_01%";
 		$tst_cmd[3] = "1.800000 1.800000 1.800000 1.800000 1.800000 1.800000 1.800000 1.800000";
 		$tst_cmd[4] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -219,9 +219,9 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
 		#!!WARN - change back to testing the ipcc file after verify
 		# !! this one is buggered by the current ssdwrap
-		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -a '-time,-lev,-lat,-lon' $in_pth_arg  gcm_T85.nc %tempf_00%";
+		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -a '-time,-lev,-lat,-lon' $in_pth_arg  gcm_T85.nc %tmp_fl_00%";
 		# ~2m on sand for gcm_T85.nc
-		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tempf_00%";  #ipcc
+		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tmp_fl_00%";  #ipcc
 		$tst_cmd[2] = "0.800000";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -237,8 +237,8 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -a 'lon,time,lev,lat' $in_pth_arg  gcm_T85.nc %tempf_00%";
-		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tempf_00%";  #ipcc
+		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -a 'lon,time,lev,lat' $in_pth_arg  gcm_T85.nc %tmp_fl_00%";
+		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tmp_fl_00%";  #ipcc
 		$tst_cmd[2] = "0.800000";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -254,8 +254,8 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -a '-lon,-time,-lev,-lat' $in_pth_arg  gcm_T85.nc %tempf_00%";
-		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tempf_00%";  #ipcc
+		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -a '-lon,-time,-lev,-lat' $in_pth_arg  gcm_T85.nc %tmp_fl_00%";
+		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tmp_fl_00%";  #ipcc
 		$tst_cmd[2] = "0.800000";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -271,8 +271,8 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -P all_new  $in_pth_arg  gcm_T85.nc %tempf_00%";
-		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tempf_00%";
+		$tst_cmd[0] = "ncpdq -h -O $fl_fmt $nco_D_flg $omp_flg -P all_new  $in_pth_arg  gcm_T85.nc %tmp_fl_00%";
+		$tst_cmd[1] = "ncks -C -H -s \"%f\" -v dopey %tmp_fl_00%";
 		$tst_cmd[2] = "0.000000";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -289,9 +289,9 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	if ($notbodi) { # too big for bodi
 		if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #			if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-			$tst_cmd[0] = "ncra -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg gcm_T85_00.nc %tempf_00%";
+			$tst_cmd[0] = "ncra -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg gcm_T85_00.nc %tmp_fl_00%";
 			# ~4m on sand.
-			$tst_cmd[1] =  "ncks -C -H -s '%f' -v d1_03   %tempf_00% ";
+			$tst_cmd[1] =  "ncks -C -H -s '%f' -v d1_03   %tmp_fl_00% ";
 			$tst_cmd[2] = "1.800001";
 			$tst_cmd[3] = "NO_SS_OK";
 			tst_run(\@tst_cmd);
@@ -313,10 +313,10 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 		####################
 		if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 	#		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-			$tst_cmd[0] = "ncrcat -h -O $fl_fmt $nco_D_flg $omp_flg -n 22,2,1 $in_pth_arg tms_lng_00.nc %tempf_00%";
-#			$tst_cmd[0] = "ncrcat -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg tms_lng_00.nc %tempf_00%";
-			$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg %tempf_00% %tempf_01%";
-			$tst_cmd[2] = "ncks -C -H -s '%f' -v PO2 %tempf_01%";
+			$tst_cmd[0] = "ncrcat -h -O $fl_fmt $nco_D_flg $omp_flg -n 22,2,1 $in_pth_arg tms_lng_00.nc %tmp_fl_00%";
+#			$tst_cmd[0] = "ncrcat -h -O $fl_fmt $nco_D_flg $omp_flg -n $fl_cnt,2,1 $in_pth_arg tms_lng_00.nc %tmp_fl_00%";
+			$tst_cmd[1] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg %tmp_fl_00% %tmp_fl_01%";
+			$tst_cmd[2] = "ncks -C -H -s '%f' -v PO2 %tmp_fl_01%";
 			$tst_cmd[3] = "12.759310";
 			$tst_cmd[4] = "SS_OK";
 			tst_run(\@tst_cmd);
@@ -333,8 +333,8 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -w lat -y sqrt -a lat,lon $in_pth_arg stl_5km.nc %tempf_00%";
-		$tst_cmd[1] = "ncks -C -H -s '%f' -v d2_02  %tempf_00%";
+		$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -w lat -y sqrt -a lat,lon $in_pth_arg stl_5km.nc %tmp_fl_00%";
+		$tst_cmd[1] = "ncks -C -H -s '%f' -v d2_02  %tmp_fl_00%";
 		$tst_cmd[2] = "1.673425";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -352,8 +352,8 @@ printf("paused @ [%s:%d]  - hit return to continue\n", __FILE__, __LINE__); my $
 	####################
 	if ($mpi_prc == 0 || ($mpi_prc > 0 && $opr_sng_mpi =~ /$opr_nm/)) {
 #		if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-		$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y rms -w lat -a lat,lon $in_pth_arg stl_5km.nc %tempf_00%";
-		$tst_cmd[1] = "ncks -C -H -s '%f' -v d2_02  %tempf_00%";
+		$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y rms -w lat -a lat,lon $in_pth_arg stl_5km.nc %tmp_fl_00%";
+		$tst_cmd[1] = "ncks -C -H -s '%f' -v d2_02  %tmp_fl_00%";
 		$tst_cmd[2] = "2.800084";
 		$tst_cmd[3] = "SS_OK";
 		tst_run(\@tst_cmd);
@@ -370,8 +370,8 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 			$dsc_sng = 'ncwa averaging all variables to scalars - gcm_T85.nc & sqt';
 			####################
 #			if ($dbg_lvl > 0) {print "\nBenchmark:  $dsc_sng\n";}
-			$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y sqrt  -w lat -a lat,lon $in_pth_arg gcm_T85.nc %tempf_00%";
-			$tst_cmd[1] = "ncks -C -H -s '%f' -v skanky  %tempf_00%";
+			$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg -y sqrt  -w lat -a lat,lon $in_pth_arg gcm_T85.nc %tmp_fl_00%";
+			$tst_cmd[1] = "ncks -C -H -s '%f' -v skanky  %tmp_fl_00%";
 			$tst_cmd[2] = "0.800000";
 			$tst_cmd[3] = "SS_OK";
 			tst_run(\@tst_cmd);
@@ -405,8 +405,8 @@ if ($dbg_lvl >= 1) {print "paused - hit return to continue"; $wait = <STDIN>;}
 # 					}
 # 				}
 				for (my $r=0; $r<2; $r++) {
-					$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg $var_sz[$r] -y sqrt  -w lat -a lat,lon $in_pth_arg gcm_T85.nc %tempf_00%";
-					$tst_cmd[1] = "ncks -C -H -s '%f' -v skanky  %tempf_00%";
+					$tst_cmd[0] = "ncwa -h -O $fl_fmt $nco_D_flg $omp_flg $var_sz[$r] -y sqrt  -w lat -a lat,lon $in_pth_arg gcm_T85.nc %tmp_fl_00%";
+					$tst_cmd[1] = "ncks -C -H -s '%f' -v skanky  %tmp_fl_00%";
 					$tst_cmd[2] = "0.800000";
 					$tst_cmd[3] = "SS_OK";
 					tst_run(\@tst_cmd);
