@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco++/ncap2_utl.cc,v 1.139 2012/01/01 20:51:54 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco++/ncap2_utl.cc,v 1.151 2013/07/24 05:02:28 zender Exp $ */
 
 /* Purpose: netCDF arithmetic processor */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -35,7 +35,6 @@ ncap_var_udf(const char *var_nm)
   // Temporary solution -- set the type to NC_INT 
   //var_ret->type=NC_INT;
 
-
   return var_ret;
 }
 
@@ -44,25 +43,29 @@ ncap_att_get
 (int var_id,
  const char *var_nm,
  const char *att_nm,
+ int location, /* I [flg] 1 - att from INPUT file  2 - att from OUTPUT file */
  prs_cls *prs_arg)
 {
-  long sz;
   int rcd;
+  int fl_id;
+  long sz;
+
   char *ln_nm;
   
   nc_type type;
   var_sct *var_ret;
   
-  rcd=nco_inq_att_flg(prs_arg->in_id,var_id,att_nm,&type,&sz);
-  if (rcd == NC_ENOTATT) 
-    return NULL_CEWI;
-  
+  if(location == 1) fl_id=prs_arg->in_id;  
+  if(location == 2) fl_id=prs_arg->out_id;  
+
+  rcd=nco_inq_att_flg(fl_id,var_id,att_nm,&type,&sz);
+  if(rcd == NC_ENOTATT) return NULL_CEWI;
   
   var_ret=(var_sct*)nco_malloc(sizeof(var_sct));
   (void)var_dfl_set(var_ret);
   
   // Make name of the form var_nm@att_nm
-  ln_nm=(char*)nco_malloc( (strlen(var_nm)+strlen(att_nm)+2)*sizeof(char));
+  ln_nm=(char *)nco_malloc((strlen(var_nm)+strlen(att_nm)+2)*sizeof(char));
   strcpy(ln_nm,var_nm);strcat(ln_nm,"@");strcat(ln_nm,att_nm);
   
   var_ret->nm=ln_nm;
@@ -76,8 +79,8 @@ ncap_att_get
   // Fill with data if NOT an initial scan
   if(!prs_arg->ntl_scn){
     var_ret->val.vp=(void *)nco_malloc(sz*nco_typ_lng(type));
-    rcd=nco_get_att(prs_arg->in_id,var_id,att_nm,var_ret->val.vp,type);
-    if (rcd !=NC_NOERR) {
+    rcd=nco_get_att(fl_id,var_id,att_nm,var_ret->val.vp,type);
+    if (rcd != NC_NOERR) {
       var_ret=nco_var_free(var_ret);
       return NULL_CEWI;
     }
@@ -114,7 +117,7 @@ ncap_att_init(             /*   [fnc] Grab an attribute from input file */
       return NULL_CEWI;
   }
   
-  var_ret=ncap_att_get(var_id,var_nm.c_str(),att_nm.c_str(),prs_arg);
+  var_ret=ncap_att_get(var_id,var_nm.c_str(),att_nm.c_str(),1,prs_arg);
   
   return var_ret;
 }
@@ -178,6 +181,7 @@ int
 ncap_att_gnrl
 (const std::string s_dst,
  const std::string s_src,
+ int location,         /*   I [flg] 1 - att from INPUT file  2 - att from OUTPUT file */
  prs_cls  *prs_arg
  ){
   int idx;
@@ -186,6 +190,7 @@ ncap_att_gnrl
   int rcd;
   int var_id; 
   int nbr_att;
+  int fl_id;
   char att_nm[NC_MAX_NAME]; 
   const char *tmp_att_nm;
   const char scl_fct_sng[]="scale_factor"; /* [sng] Unidata standard string for scale factor */
@@ -202,19 +207,22 @@ ncap_att_gnrl
   NcapVarVector &var_vtr=prs_arg->var_vtr;
   NcapVarVector att_vtr; //hold new attributes.
   
+  if(location == 1 ) fl_id=prs_arg->in_id;  
+  if(location == 2 ) fl_id=prs_arg->out_id;  
+
 
   // get var_id
-  rcd=nco_inq_varid_flg(prs_arg->in_id,s_src.c_str(),&var_id);
+  rcd=nco_inq_varid_flg(fl_id,s_src.c_str(),&var_id);
   
   if(rcd == NC_NOERR){
-    (void)nco_inq_varnatts(prs_arg->in_id,var_id,&nbr_att);
+    (void)nco_inq_varnatts(fl_id,var_id,&nbr_att);
     // loop though attributes
     for(idx=0; idx <nbr_att ; idx++){
-      (void)nco_inq_attname(prs_arg->in_id,var_id,idx,att_nm);
+      (void)nco_inq_attname(fl_id,var_id,idx,att_nm);
       //skip missing value, scale_factor , add_offset
       if(!strcmp(att_nm,nco_mss_val_sng_get()) || !strcmp(att_nm,scl_fct_sng) || !strcmp(att_nm,add_fst_sng))
         continue;
-      var_att=ncap_att_get(var_id,s_src.c_str(),att_nm,prs_arg);
+      var_att=ncap_att_get(var_id,s_src.c_str(),att_nm,location,prs_arg);
       // now add to list( change the name!!)
       if(var_att){ 
 	s_fll=s_dst+"@"+std::string(att_nm);
@@ -275,10 +283,10 @@ ncap_att_cpy
   int nbr_att=0;
   //Don't propagate if s_src is a tree-parser generated var
   if(prs_arg->ATT_PROPAGATE && s_dst != s_src && s_src[0]!='~' )
-    nbr_att=ncap_att_gnrl(s_dst,s_src,prs_arg);
+    nbr_att=ncap_att_gnrl(s_dst,s_src,1,prs_arg);
   
   if(prs_arg->ATT_INHERIT)
-    nbr_att=ncap_att_gnrl(s_dst,s_dst,prs_arg);
+    nbr_att=ncap_att_gnrl(s_dst,s_dst,1,prs_arg);
   
   return nbr_att;
 }
@@ -669,9 +677,10 @@ dmn_vtr=(nc_id==prs_arg->in_id ? prs_arg->dmn_in_vtr: prs_arg->dmn_out_vtr);
   // do error checking 
   if(!prs_arg->FORTRAN_IDX_CNV){ 
     std::ostringstream os;
-  
+    if(srt<0) srt+=cnt_dmn-1;       
+    if(end<0) end+=cnt_dmn-1;       
     if(lmt_ptr->is_usr_spc_min && (srt<0 || srt>cnt_dmn-1)) {
-
+           
       os<<"Lower limit " <<srt<<" for dim "<<lmt_ptr->nm<<" is outside range "<<0L<<"-"<<cnt_dmn-1;
       err_prn(fnc_nm,os.str());
     }
@@ -715,16 +724,18 @@ nco_var_lst_copy      /* [fnc] Purpose: Copy xtr_lst and return new list */
   for(idx=0;idx<lst_nbr;idx++){
     xtr_new_lst[idx].nm=(char *)strdup(xtr_lst[idx].nm);
     xtr_new_lst[idx].id=xtr_lst[idx].id;
+
+
   } /* end loop over variable */
   return xtr_new_lst;           
 } /* end nco_var_lst_copy() */
 
 nm_id_sct *             /* O [sct] New list */
-nco_var_lst_sub(
-		nm_id_sct *xtr_lst,     /* I [sct] input list */   
-		int *nbr_xtr,           /* I/O [ptr] size of xtr_lst and new list */
-		nm_id_sct *xtr_lst_b,   /* I [sct] list to be subtracted */   
-		int nbr_lst_b)          /* I [nbr] size eof xtr_lst_b */ 
+nco_var_lst_sub
+(nm_id_sct *xtr_lst,     /* I [sct] input list */   
+ int *xtr_nbr,           /* I/O [ptr] size of xtr_lst and new list */
+ nm_id_sct *xtr_lst_b,   /* I [sct] list to be subtracted */   
+ int nbr_lst_b)          /* I [nbr] size eof xtr_lst_b */ 
 {
   /* Purpose: Subtract from xtr_lst any elements from xtr_lst_b which are present and return new list */
   int idx;
@@ -735,68 +746,79 @@ nco_var_lst_sub(
   
   nm_id_sct *xtr_new_lst=NULL;
   
-  if(*nbr_xtr == 0) return xtr_lst;
+  if(*xtr_nbr == 0) return xtr_lst;
   
-  xtr_new_lst=(nm_id_sct*)nco_malloc((size_t)(*nbr_xtr)*sizeof(nm_id_sct)); 
-  for(idx=0;idx<*nbr_xtr;idx++){
+  xtr_new_lst=(nm_id_sct*)nco_malloc((size_t)(*xtr_nbr)*sizeof(nm_id_sct)); 
+  for(idx=0;idx<*xtr_nbr;idx++){
     match=False;
     for(xtr_idx=0;xtr_idx<nbr_lst_b;xtr_idx++)
       if(!strcmp(xtr_lst[idx].nm,xtr_lst_b[xtr_idx].nm)){match=True;break;}
     if(match) continue;
     xtr_new_lst[xtr_nbr_new].nm=(char *)strdup(xtr_lst[idx].nm);
-    xtr_new_lst[xtr_nbr_new++].id=xtr_lst[idx].id;
+    xtr_new_lst[xtr_nbr_new].id=xtr_lst[idx].id;
+
+
+    /* Increment */
+    xtr_nbr_new++;
+
+
   } /* end loop over idx */
   /* realloc to actual size */
   xtr_new_lst=(nm_id_sct*)nco_realloc(xtr_new_lst,xtr_nbr_new*sizeof(nm_id_sct)); 
   /* free old list */
-  xtr_lst=nco_nm_id_lst_free(xtr_lst,*nbr_xtr);
+  xtr_lst=nco_nm_id_lst_free(xtr_lst,*xtr_nbr);
   
-  *nbr_xtr=xtr_nbr_new;
+  *xtr_nbr=xtr_nbr_new;
   return xtr_new_lst;     
 }/* end nco_var_lst_sub */
 
 nm_id_sct *            /* O [sct] -- new list */
 nco_var_lst_add
 (nm_id_sct *xtr_lst,    /* I [sct] input list */ 
- int *nbr_xtr,          /* I/O [ptr] -- size of xtr_lst & new output list */ 
+ int *xtr_nbr,          /* I/O [ptr] -- size of xtr_lst & new output list */ 
  nm_id_sct *xtr_lst_a,  /* I [sct] list of elemenst to be added to new list */
  int nbr_lst_a)         /* I [nbr] size of xtr_lst_a */
 {
   /* Purpose: Add to xtr_lst any elements from xtr_lst_a not already present and return new list */
   int idx;
   int xtr_idx;
-  int nbr_xtr_crr;
+  int xtr_nbr_crr;
   
   nm_id_sct *xtr_new_lst;
   
   nco_bool match;
   
-  nbr_xtr_crr=*nbr_xtr;
-  if(nbr_xtr_crr > 0){
-    xtr_new_lst=(nm_id_sct*)nco_malloc((size_t)(*nbr_xtr)*sizeof(nm_id_sct));
-    for(idx=0;idx<nbr_xtr_crr;idx++){
+  xtr_nbr_crr=*xtr_nbr;
+  if(xtr_nbr_crr > 0){
+    xtr_new_lst=(nm_id_sct*)nco_malloc((size_t)(*xtr_nbr)*sizeof(nm_id_sct));
+    for(idx=0;idx<xtr_nbr_crr;idx++){
       xtr_new_lst[idx].nm=(char *)strdup(xtr_lst[idx].nm);
       xtr_new_lst[idx].id=xtr_lst[idx].id;
+
+
     } /* end loop over variables */
   }else{
-    *nbr_xtr=nbr_lst_a;
+    *xtr_nbr=nbr_lst_a;
     return nco_var_lst_copy(xtr_lst_a,nbr_lst_a);
   }/* end if */
   
   for(idx=0;idx<nbr_lst_a;idx++){
     match=False;
-    for(xtr_idx=0;xtr_idx<*nbr_xtr;xtr_idx++)
+    for(xtr_idx=0;xtr_idx<*xtr_nbr;xtr_idx++)
       if(!strcmp(xtr_lst[xtr_idx].nm,xtr_lst_a[idx].nm)){match=True;break;}
     if(match) continue;
-    xtr_new_lst=(nm_id_sct *)nco_realloc(xtr_new_lst,(size_t)(nbr_xtr_crr+1)*sizeof(nm_id_sct));
-    xtr_new_lst[nbr_xtr_crr].nm=(char *)strdup(xtr_lst_a[idx].nm);
-    xtr_new_lst[nbr_xtr_crr++].id=xtr_lst_a[idx].id;
+    xtr_new_lst=(nm_id_sct *)nco_realloc(xtr_new_lst,(size_t)(xtr_nbr_crr+1)*sizeof(nm_id_sct));
+    xtr_new_lst[xtr_nbr_crr].nm=(char *)strdup(xtr_lst_a[idx].nm);
+    xtr_new_lst[xtr_nbr_crr].id=xtr_lst_a[idx].id;
+
+
+    xtr_nbr_crr++;
+
   } /* end for */
-  *nbr_xtr=nbr_xtr_crr;
+  *xtr_nbr=xtr_nbr_crr;
   return xtr_new_lst;           
 } /* end nco_var_lst_add */
 
-
 nm_id_sct * /* O [sct] List of dimensions associated with input variable list */
 nco_dmn_lst /* [fnc] Create list of all dimensions in file  */
 (const int nc_id, /* I [id] netCDF input-file ID */
@@ -815,6 +837,9 @@ nco_dmn_lst /* [fnc] Create list of all dimensions in file  */
     (void)nco_inq_dimname(nc_id,idx,dmn_nm);
     dmn[idx].id=idx;
     dmn[idx].nm=(char *)strdup(dmn_nm);
+
+    
+
   } /* end loop over dmn */
   
   *nbr_dmn=nbr_dmn_in;
@@ -839,24 +864,28 @@ nco_att_lst_mk
     // Check for attribute
     if( var_vtr[idx]->xpr_typ !=ncap_att) continue;
     (void)strcpy(var_nm, var_vtr[idx]->getVar().c_str());
-    
+
     rcd=nco_inq_varid_flg(out_id,var_nm,&var_id);
     if(rcd== NC_NOERR) continue;   
     rcd=nco_inq_varid_flg(in_id,var_nm,&var_id);   
     if(rcd == NC_NOERR){
       /* eliminate any duplicates from list */
       for(jdx=0;jdx<size;jdx++)
-	if(!strcmp(xtr_lst[jdx].nm,var_nm)) break;
+        if(!strcmp(xtr_lst[jdx].nm,var_nm)) break;
       if(jdx!=size) continue;
       /* fxm mmr TODO 491: memory leak xtr_lst */
       xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,(size+1)*sizeof(nm_id_sct));
       xtr_lst[size].id=var_id;
-      xtr_lst[size++].nm=(char *)strdup(var_nm);
+      xtr_lst[size].nm=(char *)strdup(var_nm); 
+
+      /* Increment */
+      size++;
+
     } /* end if */
   } /* end loop over att */
-  
+
   *nbr_lst=size;
-  
+
   return xtr_lst;
 } /* end nco_att_lst_mk() */
 
@@ -1181,27 +1210,9 @@ ncap_def_dim(
 	     prs_cls *prs_arg){
   const char fnc_nm[]="ncap_def_dim"; 
   
-  // int idx;
-  // char ch; 
-  int len;
-  
   dmn_sct *dmn_nw;             
   dmn_sct *dmn_in_e;
   dmn_sct *dmn_out_e;
-  
-  
-  
-  len=dmn_nm.length();
-  // Ckeck for a valid name 
-  /*
-  for(idx=0 ; idx < len ; idx++){ 
-    ch=dmn_nm[idx];
-    if( !( isalpha(ch) || isdigit(ch) || ch=='.' || ch=='_' ||ch=='-') ){ 
-      wrn_prn(fnc_nm,"dim \""+ dmn_nm + "\" - Invalid dimension name.");
-      return False;;
-    }
-  }         
-  */
 
   // Check if dimension already exists
   dmn_in_e=prs_arg->dmn_in_vtr.find(dmn_nm);
@@ -1959,7 +1970,7 @@ ncap_cst_mk( /* [fnc] create casting var from a list of dims */
 
 #ifdef _OPENMP
        if( omp_in_parallel())
-	 err_prn(fnc_nm,"Attempt to go into netcdf define mode while in OPENMP parallel mode");
+	 err_prn(fnc_nm,"Attempt to go into netCDF define mode while in OpenMP parallel mode");
       
 #endif
 
@@ -1993,6 +2004,7 @@ ncap_cst_mk( /* [fnc] create casting var from a list of dims */
   /* Overwrite with LHS information */
   /* fxm mmr: memory leak with var->nm */
   var->nm=(char *)strdup(tpl_nm);
+  var->nm_fll=NULL;
   var->type=NC_DOUBLE;
   var->nbr_dim=dmn_nbr;
   var->dim=dmn;
@@ -2055,7 +2067,7 @@ ncap_cst_mk( /* [fnc] create casting var from a list of dims */
   } /* end scope */
  end_var:
   
-  if(dbg_lvl_get() > 2) {
+  if(dbg_lvl_get() >= nco_dbg_scl) {
     std::ostringstream os;
     os<<"creating LHS cast template var->nm " <<var->nm <<" var->nbr_dim " <<var->nbr_dim <<" var->sz " <<var->sz; 
     wrn_prn(fnc_nm,os.str());
@@ -2102,7 +2114,7 @@ ncap_cst_do(
       var=var_tmp;
     }
     
-    if(dbg_lvl_get() > 2){
+    if(dbg_lvl_get() >= nco_dbg_scl){
       std::ostringstream os;
       os<<"Stretching variable "<<var->nm << "with LHS template var->nm "<<var_cst->nm <<"var->nbr_dim " <<var_cst->nbr_dim; 
       os<<" var->sz " <<var_cst->sz;
@@ -2373,7 +2385,7 @@ ncap_get_var_mem(
   char *cp_lcl;
   
   
-  if(dbg_lvl_get() > 2){
+  if(dbg_lvl_get() >= nco_dbg_scl){
     std::ostringstream os;
     os<<"Depth=" << dpt<<" "<<dmn_vtr[dpt]->nm<<" "<<srt<<" "<<end<<" "<<cnt<<" "<<srd;
     dbg_prn(fnc_nm,os.str());
@@ -2537,7 +2549,7 @@ char *cp_out)                   // Slab to be "put"
   char *cp_end;
   
     
-  if(dbg_lvl_get() > 2){
+  if(dbg_lvl_get() >= nco_dbg_scl){
     std::ostringstream os;
     os<<"Depth=" << dpt<<" "<<dmn_vtr[dpt]->nm<<" "<<srt<<" "<<end<<" "<<cnt<<" "<<srd<<" "<<slb_sz<<" nbr_lpp="<<nbr_lpp;
     dbg_prn(fnc_nm,os.str());
@@ -2889,7 +2901,7 @@ int ncap_mpi_srt(
   
   
   // populate exp_vtr;
-  if(dbg_lvl_get() > 0)
+  if(dbg_lvl_get() >= nco_dbg_std)
     dbg_prn(fnc_nm,"Start");
   
   
@@ -3030,7 +3042,7 @@ int ncap_mpi_srt(
       std::cout <<"-------------------------------\n";
     } //end idx
   }
-  if(dbg_lvl_get() > 0) dbg_prn(fnc_nm,"End");       
+  if(dbg_lvl_get() >= nco_dbg_std) dbg_prn(fnc_nm,"End");       
   
   return 0;
 }
