@@ -3,23 +3,23 @@
 
 #include <antlr/config.hpp>
 #include "ncoParserTokenTypes.hpp"
-/* $ANTLR 2.7.7 (20110618): "ncoGrammer.g" -> "ncoTree.hpp"$ */
+/* $ANTLR 2.7.7 (2006-11-01): "ncoGrammer.g" -> "ncoTree.hpp"$ */
 #include <antlr/TreeParser.hpp>
 
 #line 1 "ncoGrammer.g"
 
-/* $Header: /cvsroot/nco/nco/src/nco++/ncoTree.hpp,v 1.111 2012/02/07 07:12:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco++/ncoTree.hpp,v 1.137 2013/09/22 15:25:48 zender Exp $ */
 
 /* Purpose: ANTLR Grammar and support files for ncap2 */
 
-/* Copyright (C) 2005--2012 Charlie Zender
+/* Copyright (C) 2005--2012 Henry Butowsky and Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
     // C Standard Headers
     #include <assert.h>
     #include <ctype.h>
-    #include <malloc.h>
+	#include <stdlib.h>
     #include <math.h>
     #if !(defined __xlC__) && !(defined SGIMP64) // C++ compilers that do not allow stdint.h
     #include <stdint.h> // Required by g++ for LLONG_MAX, ULLONG_MAX, by icpc for int64_t    
@@ -50,7 +50,7 @@
 #line 51 "ncoTree.hpp"
 class CUSTOM_API ncoTree : public ANTLR_USE_NAMESPACE(antlr)TreeParser, public ncoParserTokenTypes
 {
-#line 645 "ncoGrammer.g"
+#line 619 "ncoGrammer.g"
 
 
 private:
@@ -61,9 +61,6 @@ public:
     prs_cls *prs_arg;
     ASTFactory myFactory;
 
-
-
-
      //Structure to hold AST pointers to indices in hyperslabs -only temporary 
      typedef struct{
         ANTLR_USE_NAMESPACE(antlr)RefAST ind[3];
@@ -110,7 +107,6 @@ vector<ast_lmt_sct> &ast_lmt_vtr)
          hyp.ind[0]=ANTLR_USE_NAMESPACE(antlr)nullAST;
          hyp.ind[1]=ANTLR_USE_NAMESPACE(antlr)nullAST;
          hyp.ind[2]=ANTLR_USE_NAMESPACE(antlr)nullAST;
-
              
        if(lRef->getType()!=LMT) 
             return 0;
@@ -161,7 +157,6 @@ vector<ast_lmt_sct> &ast_lmt_vtr)
        lRef=lRef->getNextSibling();
      }
      return nbr_dmn;
-
 } 
 
 int 
@@ -172,50 +167,19 @@ NcapVector<lmt_sct*> &lmt_vtr )
 int nbr_dmn;
 int idx;
 int jdx;
-long lcl_ind[3];
-
-var_sct *var_out;
 lmt_sct *lmt_ptr;
 RefAST aRef;
 
 vector<ast_lmt_sct> ast_lmt_vtr;
 
-
 // populate ast_lmt_vtr
 nbr_dmn=lmt_init(lmt,ast_lmt_vtr);
 
   for(idx=0 ; idx <nbr_dmn ; idx++){
 
-
-     lcl_ind[0]=-2; lcl_ind[1]=-2; lcl_ind[2]=0; 
-
-    for(jdx=0 ; jdx <3 ; jdx++){
-
-     aRef=ast_lmt_vtr[idx].ind[jdx];
-
-     if(!aRef)
-        continue; //do nothing - use default lcl_ind values     
-     else if( aRef->getType() == COLON){
-       if(jdx <2) lcl_ind[jdx]=-1;
-     }else{
-         // Calculate number using out()
-         var_out=out(aRef);
-
-         // convert result to type int
-          var_out=nco_var_cnf_typ(NC_INT,var_out);    
-         (void)cast_void_nctype((nc_type)NC_INT,&var_out->val);
-
-          // only interested in the first value.
-         lcl_ind[jdx]=var_out->val.ip[0];
-
-         var_out=nco_var_free(var_out);
-        }
-     }// end jdx
-         
      // fill out lmt structure
      // use same logic as nco_lmt_prs 
      lmt_ptr=(lmt_sct*)nco_calloc((size_t)1,sizeof(lmt_sct));
-
      lmt_ptr->nm=NULL;
      //lmt_ptr->lmt_typ=-1;
      lmt_ptr->is_usr_spc_lmt=True; /* True if any part of limit is user-specified, else False */
@@ -224,36 +188,51 @@ nbr_dmn=lmt_init(lmt,ast_lmt_vtr);
      lmt_ptr->srd_sng=NULL;
      lmt_ptr->is_usr_spc_min=False;
      lmt_ptr->is_usr_spc_max=False;
+     /* rec_skp_ntl_spf is used for record dimension in multi-file operators */
+     lmt_ptr->rec_skp_ntl_spf=0L; /* Number of records skipped in initial superfluous files */
 
-     /* rec_skp_nsh_spf is used for record dimension in multi-file operators */
-     lmt_ptr->rec_skp_nsh_spf=0L; /* Number of records skipped in initial superfluous files */
-    
-    /* Fill-in structure */
-    if( lcl_ind[0] >= 0){ 
-           lmt_ptr->is_usr_spc_min=True;
-           lmt_ptr->srt=lcl_ind[0]; 
-    }    
+    for(jdx=0 ; jdx <3 ; jdx++){
+      long ldx=0L;
+      var_sct *var_out;
 
-    /* Fill-in structure */
-    if( lcl_ind[1] >= 0) {
-           lmt_ptr->is_usr_spc_max=True;
-           lmt_ptr->end=lcl_ind[1];
-    }    
-
-    /* Fill-in structure */
-    if( lcl_ind[2] > 0) {
-           lmt_ptr->srd_sng=strdup("~fill_in");
-           lmt_ptr->srd=lcl_ind[2];
-    }    
+      aRef=ast_lmt_vtr[idx].ind[jdx];
 
+      if(aRef && aRef->getType() != COLON ){
+        // Calculate number using out()
+        var_out=out(aRef);
+        // convert result to type int
+        var_out=nco_var_cnf_typ(NC_INT,var_out);    
+        (void)cast_void_nctype((nc_type)NC_INT,&var_out->val);
+         // only interested in the first value.
+        ldx=var_out->val.ip[0];
+        var_out=nco_var_free(var_out);
+        
+        // switch jdx 0-srt,1-end,2-srd
+        switch(jdx){
+          case 0: 
+             lmt_ptr->is_usr_spc_min=True;
+             lmt_ptr->srt=ldx;
+             break;
+          case 1: //end
+             lmt_ptr->is_usr_spc_max=True;
+             lmt_ptr->end=ldx;
+             break;
+          case 2: //srd
+             lmt_ptr->srd_sng=strdup("~fill_in");
+             lmt_ptr->srd=ldx;         
+             break;
+        }
+      }
+    }// end jdx
+         
     /* need to deal with situation where only start is defined -- ie picking only a single value */
-    if(lcl_ind[0] >=0 && lcl_ind[1]==-2){
-          lmt_ptr->is_usr_spc_max=True;
-          lmt_ptr->end=lcl_ind[0]; 
+    if( lmt_ptr->is_usr_spc_min==True && lmt_ptr->is_usr_spc_max==False && lmt_ptr->srd_sng==NULL){
+        lmt_ptr->is_usr_spc_max=True;
+        lmt_ptr->end=lmt_ptr->srt; 
     }    
 
     lmt_vtr.push_back(lmt_ptr);
-   } // end idx
+  } // end idx
 
    return nbr_dmn;
 } /* end lmt_mk */
@@ -277,7 +256,7 @@ public:
      ncap_omp_exe(
      std::vector< std::vector<RefAST> > &all_ast_vtr,
      ncoTree** wlk_ptr,
-     int nbr_wlk);
+     int wlk_nbr);
 
      if(tr== ANTLR_USE_NAMESPACE(antlr)nullAST)
         err_prn("run_dbl"," REPORTS given a null AST Refrence\n");
@@ -323,8 +302,6 @@ public:
     goto end;
     } //end if
 
-
-
 small: 
      idx=0;
      ntr=tr;
@@ -459,10 +436,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 113;
+	static const int NUM_TOKENS = 114;
 #else
 	enum {
-		NUM_TOKENS = 113
+		NUM_TOKENS = 114
 	};
 #endif
 	
