@@ -1,6 +1,6 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_aux.c,v 1.37 2012/02/13 23:09:50 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_aux.c,v 1.45 2013/09/20 19:52:53 pvicente Exp $ */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -17,7 +17,7 @@
 
 #include "nco_aux.h" /* Auxiliary coordinates */
 
-int 
+nco_bool 
 nco_find_lat_lon
 (int nc_id,
  char var_nm_lat[], 
@@ -50,55 +50,58 @@ nco_find_lat_lon
 
   long lenp;
 
-  nc_type var_type; /* [enm] variable type */
-  
+  nc_type var_typ; /* [enm] variable type */
+
   /* Make sure CF tag exists. Currently require CF-1.0 value */
   if(NCO_GET_ATT_CHAR(nc_id,NC_GLOBAL,"Conventions",value) || !strstr(value,"CF-1.0")){
+    if(dbg_lvl_get() >= nco_dbg_dev)
     (void)fprintf(stderr,"%s: WARNING %s reports file \"Convention\" attribute is missing or not equal to \"CF-1.0\". Auxiliary coordinate support (i.e., the -X option) cannot be expected to behave well file does not support CF-1.0 metadata conventions. Continuing anyway...\n",prg_nm_get(),fnc_nm);
   } /* !CF */
-  
+
   /* Get number of variables */
   rcd=nco_inq_nvars(nc_id,&nvars);
-  
+
   /* For each variable, see if standard name is latitude or longitude */
   for(idx=0;idx<nvars && ret<2;idx++){
-    nco_inq_var(nc_id,idx,var_nm,&var_type,&var_dmn_nbr,var_dimid,&var_att_nbr);
+    nco_inq_var(nc_id,idx,var_nm,&var_typ,&var_dmn_nbr,var_dimid,&var_att_nbr);
     lenp=0;
     if(!nco_inq_attlen_flg(nc_id,idx,"standard_name",&lenp)){
       NCO_GET_ATT_CHAR(nc_id,idx,"standard_name",value);
       value[lenp]='\0';
       if(strcmp(value,"latitude") == 0){
-	strcpy(var_nm_lat,var_nm);
-	*lat_id=idx;
-	
-	/* Get units; assume same for both lat and lon */
-	rcd=nco_inq_attlen(nc_id,idx,"units",&lenp);
-	if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_find_lat_lon() reports CF convention requires \"latitude\" to have units attribute\n");
-	*units=(char *)nco_malloc((lenp+1L)*sizeof(char *));
-	NCO_GET_ATT_CHAR(nc_id,idx,"units",*units);
-	units[lenp]='\0';
-	
-	if(var_dmn_nbr > 1) (void)fprintf(stderr,"%s: WARNING %s reports latitude variable %s has %d dimensions. NCO only supports hyperslabbing of auxiliary coordinate variables with a single dimension. Continuing with unpredictable results...\n",prg_nm_get(),fnc_nm,var_nm,var_dmn_nbr);
+        strcpy(var_nm_lat,var_nm);
+        *lat_id=idx;
 
-	/* Assign type; assumed same for both lat and lon */
-	*crd_typ=var_type;
-	ret++;
+        /* Get units; assume same for both lat and lon */
+        rcd=nco_inq_attlen(nc_id,idx,"units",&lenp);
+        if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_find_lat_lon() reports CF convention requires \"latitude\" to have units attribute\n");
+        *units=(char *)nco_malloc((lenp+1L)*sizeof(char *));
+        NCO_GET_ATT_CHAR(nc_id,idx,"units",*units);
+        units[lenp]='\0';
+
+        if(var_dmn_nbr > 1) (void)fprintf(stderr,"%s: WARNING %s reports latitude variable %s has %d dimensions. NCO only supports hyperslabbing of auxiliary coordinate variables with a single dimension. Continuing with unpredictable results...\n",prg_nm_get(),fnc_nm,var_nm,var_dmn_nbr);
+
+        /* Assign type; assumed same for both lat and lon */
+        *crd_typ=var_typ;
+        ret++;
       } /* end if var is lattitude */
-      
+
       if(strcmp(value,"longitude") == 0){
-	strcpy(var_nm_lon,var_nm);
-	*lon_id=idx;
-	ret++;
+        strcpy(var_nm_lon,var_nm);
+        *lon_id=idx;
+        ret++;
       } /* end if var is longitude */
-      
+
     } /* end if standard_name */
-    
+
   } /* end loop over vars */
-  
-  /* Die if both not found */
-  if(ret != 2) nco_err_exit(rcd,"nco_find_lat_lon() unable to identify lat/lon auxiliary coordinate variables.");
- 
-  return rcd;
+
+
+  if(ret != 2){
+    if(dbg_lvl_get() >= nco_dbg_dev) (void)fprintf(stdout,"nco_find_lat_lon() unable to identify lat/lon auxiliary coordinate variables.\n");
+    return False;
+  } else return True;
+
 
 } /* end nco_find_lat_lon() */
 
@@ -115,13 +118,13 @@ nco_get_dmn_info
   
   int rcd=NC_NOERR;
   
-  nc_type var_type;                   /* variable type */
+  nc_type var_typ;                   /* variable type */
   int var_dimid[NC_MAX_VAR_DIMS];    /* dimension ids */
   int var_att_nbr;                      /* number of attributes */
   int var_dmn_nbr;                      /* number of dims */
   
   /* Get dimension information */
-  rcd=nco_inq_var(nc_id,var_id,0,&var_type,&var_dmn_nbr,var_dimid,&var_att_nbr);
+  rcd=nco_inq_var(nc_id,var_id,0,&var_typ,&var_dmn_nbr,var_dimid,&var_att_nbr);
   if(rcd == NC_NOERR){
     *dimid=var_dimid[0];
     rcd=nco_inq_dimlen(nc_id,var_dimid[0],dmn_sz);
@@ -144,7 +147,7 @@ nco_aux_evl
      match rectangular region specified by -X arguments.
      Intended for use with non-monotonic grids
      Requires CF-1.0 conventions
-     Uses lat/lon centers rather than cell_bounds to detect matches
+     Uses latitude/longitude centers rather than cell_bounds to detect matches
      Code assumes units are degrees if they are not radians */
   
   const char fnc_nm[]="nco_aux_evl()"; /* [sng] Function name */
@@ -177,7 +180,7 @@ nco_aux_evl
   int lon_id;
   int rcd=NC_NOERR;
   
-  lmt_sct **lmt=0; /* [sct] List of returned lmt structures */
+  lmt_sct **lmt=NULL; /* [sct] List of returned lmt structures */
   
   long dmn_sz=0;
   
@@ -185,9 +188,15 @@ nco_aux_evl
   
   void *vp_lat; /* [dgr] Latitude coordinate array, float or double */
   void *vp_lon; /* [dgr] Longitude coordinate array, float or double */
+
+  nco_bool has_lat_lon;
   
   /* Obtain lat/lon variable names */
-  rcd+=nco_find_lat_lon(in_id,var_nm_lat,var_nm_lon,&units,&lat_id,&lon_id,&crd_typ);
+  has_lat_lon=nco_find_lat_lon(in_id,var_nm_lat,var_nm_lon,&units,&lat_id,&lon_id,&crd_typ);
+
+  if (has_lat_lon == False){
+    return NULL;
+  }
   
   /* Obtain dimension information of lat/lon coordinates */
   rcd+=nco_get_dmn_info(in_id,lat_id,dmn_nm,&dmn_id,&dmn_sz);
@@ -208,21 +217,30 @@ nco_aux_evl
   
   *lmt_nbr=0;
   
-  lmt_sct base;
-  base.nm=(char *)strdup(dmn_nm);
-  base.lmt_typ=lmt_dmn_idx;
-  base.is_usr_spc_lmt=True; 
-  base.is_usr_spc_min=True; 
-  base.is_usr_spc_max=True;
-  base.srd_sng=(char *)strdup("1");
-  base.is_rec_dmn=0;
-  base.id=dmn_id;
-  base.min_idx=0;
-  base.max_idx=0;
-  base.srt=0L;
-  base.end=0L;
-  base.cnt=0L;
-  base.srd=1L;
+  lmt_sct lmt_tpl;
+  /* 20130428: Utilitze nco_lmt_init() to initialize limit structure? */
+  (void)nco_lmt_init(&lmt_tpl);
+  assert(dmn_nm);
+  lmt_tpl.nm=(char *)strdup(dmn_nm);
+  lmt_tpl.lmt_typ=lmt_dmn_idx;
+  lmt_tpl.is_usr_spc_lmt=True; 
+  lmt_tpl.is_usr_spc_min=True; 
+  lmt_tpl.is_usr_spc_max=True;
+  lmt_tpl.flg_mro=False;
+  lmt_tpl.srd_sng=(char *)strdup("1");
+  /*  lmt_tpl.drn_sng=(char *)strdup("1");*/
+  lmt_tpl.drn_sng=NULL;
+  /*  lmt_tpl.mro_sng=(char *)strdup("m");*/
+  lmt_tpl.mro_sng=NULL;
+  lmt_tpl.is_rec_dmn=0;
+  lmt_tpl.id=dmn_id;
+  lmt_tpl.min_idx=0;
+  lmt_tpl.max_idx=0;
+  lmt_tpl.srt=0L;
+  lmt_tpl.end=0L;
+  lmt_tpl.cnt=0L;
+  lmt_tpl.srd=1L;
+  lmt_tpl.drn=1L;
   
   /* malloc() lmt structure to return
      No way to know exact size in advance though maximum is about dim_sz/2 */
@@ -265,16 +283,16 @@ nco_aux_evl
       }else if(cll_idx_min != -1){
 	/* Current cell is not within bounding box though immediately previous cell is */
 	sprintf(cll_idx_sng,"%d",cll_idx_min);
-	base.min_sng=(char *)strdup(cll_idx_sng);
-	base.min_idx=base.srt=cll_idx_min;
+	lmt_tpl.min_sng=(char *)strdup(cll_idx_sng);
+	lmt_tpl.min_idx=lmt_tpl.srt=cll_idx_min;
 	sprintf(cll_idx_sng,"%d",cll_idx_min+cll_nbr_cns-1);
-	base.max_sng=(char *)strdup(cll_idx_sng);
-	base.max_idx=base.end=cll_idx_min+cll_nbr_cns-1;
-	base.cnt=cll_nbr_cns;
+	lmt_tpl.max_sng=(char *)strdup(cll_idx_sng);
+	lmt_tpl.max_idx=lmt_tpl.end=cll_idx_min+cll_nbr_cns-1;
+	lmt_tpl.cnt=cll_nbr_cns;
 	(*lmt_nbr)++;
 	if(*lmt_nbr > MAX_LMT_NBR) nco_err_exit(0,"%s: Number of slabs exceeds allocated mamory");
 	lmt[(*lmt_nbr)-1]=(lmt_sct *)nco_malloc(sizeof(lmt_sct));
-	*lmt[(*lmt_nbr)-1]=base;
+	*lmt[(*lmt_nbr)-1]=lmt_tpl;
 	cll_grp_nbr++;
 	cll_nbr_ttl+=cll_nbr_cns;
 	/* Indicate that next cell, if any, in this bounding box requires new limit structure */
