@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_csm.c,v 1.56 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_csm.c,v 1.61 2013/06/25 16:56:55 zender Exp $ */
 
 /* Purpose: CCM/CCSM/CF conventions */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -50,11 +50,11 @@ nco_cnv_ccm_ccsm_cf_inq /* O [fnc] Check if file obeys CCM/CCSM/CF conventions *
     if(strstr(att_val,"CF-1.0")) CNV_CCM_CCSM_CF=True; /* NB: Not fully implemented TODO nco145 */
     /* As of 20060514, CLM 3.0 uses CF1.0 not CF-1.0 (CAM gets it right) */
     if(strstr(att_val,"CF1.0")) CNV_CCM_CCSM_CF=True; /* NB: Not fully implemented TODO nco145 */
-    if(CNV_CCM_CCSM_CF && dbg_lvl_get() > 0){
+    if(CNV_CCM_CCSM_CF && dbg_lvl_get() >= nco_dbg_std){
       (void)fprintf(stderr,"%s: CONVENTION File \"%s\" attribute is \"%s\"\n",prg_nm_get(),cnv_sng,att_val);
       if(cnv_sng == cnv_sng_LC) (void)fprintf(stderr,"%s: WARNING: This file uses a non-standard attribute (\"%s\") to indicate the netCDF convention. The correct attribute is \"%s\".\n",prg_nm_get(),cnv_sng_LC,cnv_sng_UC);
       /* Only warn in arithmetic operators where conventions change behavior */
-      if(dbg_lvl_get() > 1 && nco_is_rth_opr(prg_get())) (void)fprintf(stderr,"%s: INFO NCO has a unified (though incomplete) treatment of many related (official and unoffical) conventions including the older CCM and CCSM and newer CF conventions. To adhere to these conventions, NCO implements variable-specific exceptions in certain operators, e.g., ncbo will not subtract variables named \"date\" or \"gw\", and many operators will always leave coordinate variables unchanged. The full list of exceptions is in the manual http://nco.sf.net/nco.html#CF\n",prg_nm_get());
+      if(dbg_lvl_get() >= nco_dbg_fl && nco_is_rth_opr(prg_get())) (void)fprintf(stderr,"%s: INFO NCO has a unified (though incomplete) treatment of many related (official and unoffical) conventions including the older CCM and CCSM and newer CF conventions. To adhere to these conventions, NCO implements variable-specific exceptions in certain operators, e.g., ncbo will not subtract variables named \"date\" or \"gw\", and many operators will always leave coordinate variables unchanged. The full list of exceptions is in the manual http://nco.sf.net/nco.html#CF\n",prg_nm_get());
     } /* endif dbg */
     att_val=(char *)nco_free(att_val);
   } /* endif */
@@ -137,7 +137,7 @@ nm_id_sct * /* O [sct] Extraction list */
 nco_cnv_cf_crd_add /* [fnc] Add coordinates defined by CF convention */
 (const int nc_id, /* I netCDF file ID */
  nm_id_sct *xtr_lst, /* I/O current extraction list (destroyed) */
- int * const nbr_xtr) /* I/O number of variables in current extraction list */
+ int * const xtr_nbr) /* I/O number of variables in current extraction list */
 {
   /* Purpose: Detect coordinates specified by CF convention and add them to extraction list
      http://www.cgd.ucar.edu/cms/eaton/cf-metadata/CF-1.0.html#grid_ex2 */
@@ -164,7 +164,7 @@ nco_cnv_cf_crd_add /* [fnc] Add coordinates defined by CF convention */
   nc_type att_typ;
   
   /* ...for each variable in extraction list... */
-  for(idx_var=0;idx_var<*nbr_xtr;idx_var++){
+  for(idx_var=0;idx_var<*xtr_nbr;idx_var++){
     /* Eschew indirection */
     var_id=xtr_lst[idx_var].id;
     /* Find number of attributes */
@@ -199,15 +199,15 @@ nco_cnv_cf_crd_add /* [fnc] Add coordinates defined by CF convention */
 	  if(rcd == NC_NOERR){
 	    /* idx_var2 labels inner loop over variables */
 	    /* Is "coordinate" already on extraction list? */
-	    for(idx_var2=0;idx_var2<*nbr_xtr;idx_var2++){
+	    for(idx_var2=0;idx_var2<*xtr_nbr;idx_var2++){
 	      if(crd_id == xtr_lst[idx_var2].id) break;
 	    } /* end loop over idx_var2 */
-	    if(idx_var2 == *nbr_xtr){
+	    if(idx_var2 == *xtr_nbr){
 	      /* Add coordinate to list */
-	      xtr_lst=(nm_id_sct *)nco_realloc((void *)xtr_lst,(*nbr_xtr+1)*sizeof(nm_id_sct));
-	      xtr_lst[*nbr_xtr].nm=(char *)strdup(crd_lst[idx_crd]);
-	      xtr_lst[*nbr_xtr].id=crd_id;
-	      (*nbr_xtr)++; /* NB: Changes size of current loop! */
+	      xtr_lst=(nm_id_sct *)nco_realloc((void *)xtr_lst,(*xtr_nbr+1)*sizeof(nm_id_sct));
+	      xtr_lst[*xtr_nbr].nm=(char *)strdup(crd_lst[idx_crd]);
+	      xtr_lst[*xtr_nbr].id=crd_id;
+	      (*xtr_nbr)++; /* NB: Changes size of current loop! */
 	      /* Continue to next coordinate in loop */
 	      continue;
 	    } /* end if coordinate was not already in list */
