@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.c,v 1.61 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.c,v 1.82 2013/07/09 18:10:59 zender Exp $ */
 
 /* Purpose: List utilities */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -16,25 +16,41 @@
 /* Compile following routines only if regular expression library is present */
 #ifdef NCO_HAVE_REGEX_FUNCTIONALITY
 
-int /* O [nbr] Number of matches found */
-nco_lst_meta_search /* [fnc] Search for pattern matches in var string list */
-(int var_nbr_all, /* I [nbr] Size of var_lst_all and var_xtr_rqs */
- nm_id_sct *var_lst_all, /* I [sct] All variables in input file (with IDs) */
- char *rx_sng, /* I [sng] Regular expression pattern */
- nco_bool *var_xtr_rqs) /* O [flg] Matched variable holder */
+char * /* O [nbr] Format string with printf()-formats replaced */
+nco_fmt_sng_printf_subst /* [fnc] Replace printf() format statements */
+(const char * const fmt_sng) /* I [sng] Format string before processing */
 {
-  /* Purpose: Match regular expressions to members of list */
+  /* Purpose: Replace printf()-format statements with string format statement */
+
+  const char fnc_nm[]="nco_fmt_sng_printf_subst()"; /* [sng] Function name */
+
+  /* Regular expressions for printf()-formats
+     const char rx_sng[]="%(?:\\d+\\$)?[+-]?(?:[ 0]|'.{1})?-?\\d*(?:\\.\\d+)?[bcdeEufFgGosxX]"; [sng] Regular expression pattern found on Web 
+     const char rx_sng[]="%([+- 0#])?([0-9]*)?([\.?[0-9]*)?[bcdeEfFgGiosuxX]"; Space in position 6 causes error
+     Flag character " " crashes everywhere
+     Flag characters "'I" are fancy
+     Length modifiers "jzt" are somewhat fancy (unlikely to be on Windows) */
+     
+#ifdef _MSC_VER
+  const char rx_sng[]="%([+-0#])?([0-9]*)?([\.?[0-9]*)?([h+l+L])?[bcdeEfFgGiosuxX]"; /* [sng] Regular expression pattern */
+#else /* _MSC_VER */
+  const char rx_sng[]="%([+-0#'I])?([0-9]*)?([\.?[0-9]*)?([h+l+Ljzt])?[bcdeEfFgGiosuxX]"; /* [sng] Regular expression pattern */
+#endif /* _MSC_VER */
+
+  char *fmt_sng_new;
 
-  int idx;
   int err_id;
   int flg_cmp; /* Comparison flags */
   int flg_exe; /* Execution flages */
   int mch_nbr=0;
-  size_t nbr_sub_xpr;
-  
+
   regmatch_t *result;
   regex_t *rx;
+  regoff_t mch_psn_srt=0; /* Byte offset from start of string to start of substring */
+  regoff_t mch_psn_end=0; /* Byte offset from start of string to first character after substring */
 
+  size_t rx_prn_sub_xpr_nbr;
+  
   rx=(regex_t *)nco_malloc(sizeof(regex_t));
 
   /* Choose RE_SYNTAX_POSIX_EXTENDED regular expression type */
@@ -47,33 +63,113 @@ nco_lst_meta_search /* [fnc] Search for pattern matches in var string list */
     char const * rx_err_sng;  
     /* POSIX regcomp return error codes */
     switch(err_id){
-    case REG_BADPAT: rx_err_sng="Invalid pattern."; break;  
-    case REG_ECOLLATE: rx_err_sng="Not implemented."; break;
-    case REG_ECTYPE: rx_err_sng="Invalid character class name."; break;
-    case REG_EESCAPE: rx_err_sng="Trailing backslash."; break;
-    case REG_ESUBREG: rx_err_sng="Invalid back reference."; break;
-    case REG_EBRACK: rx_err_sng="Unmatched left bracket."; break;
-    case REG_EPAREN: rx_err_sng="Parenthesis imbalance."; break;
-    case REG_EBRACE: rx_err_sng="Unmatched {."; break;
-    case REG_BADBR: rx_err_sng="Invalid contents of { }."; break;
-    case REG_ERANGE: rx_err_sng="Invalid range end."; break;
-    case REG_ESPACE: rx_err_sng="Ran out of memory."; break;
+    case REG_BADPAT: rx_err_sng="Invalid pattern"; break;  
+    case REG_ECOLLATE: rx_err_sng="Not implemented"; break;
+    case REG_ECTYPE: rx_err_sng="Invalid character class name"; break;
+    case REG_EESCAPE: rx_err_sng="Trailing backslash"; break;
+    case REG_ESUBREG: rx_err_sng="Invalid back reference"; break;
+    case REG_EBRACK: rx_err_sng="Unmatched left bracket"; break;
+    case REG_EPAREN: rx_err_sng="Parenthesis imbalance"; break;
+    case REG_EBRACE: rx_err_sng="Unmatched {"; break;
+    case REG_BADBR: rx_err_sng="Invalid contents of { }"; break;
+    case REG_ERANGE: rx_err_sng="Invalid range end"; break;
+    case REG_ESPACE: rx_err_sng="Ran out of memory"; break;
     case REG_BADRPT: rx_err_sng="No preceding re for repetition op"; break;
-    default: rx_err_sng="Invalid pattern."; break;  
+    default: rx_err_sng="Invalid pattern"; break;  
     } /* end switch */
-    (void)fprintf(stdout,"%s: ERROR nco_lst_meta_search() error in regular expression \"%s\" %s \n",prg_nm_get(),rx_sng,rx_err_sng); 
+    (void)fprintf(stdout,"%s: ERROR %s reports error in regular expression \"%s\" %s.\n",prg_nm_get(),fnc_nm,rx_sng,rx_err_sng); 
     nco_exit(EXIT_FAILURE);
   } /* end if err */
 
-  nbr_sub_xpr=rx->re_nsub+1; /* How many matches are there in line? */
+  rx_prn_sub_xpr_nbr=rx->re_nsub+1; /* Number of parenthesized sub-expressions */
 
   /* Search string */
-  result=(regmatch_t *)nco_malloc(sizeof(regmatch_t)*nbr_sub_xpr);
+  result=(regmatch_t *)nco_malloc(sizeof(regmatch_t)*rx_prn_sub_xpr_nbr);
 
-  /* Search each variable string for matches */
-  for(idx=0;idx<var_nbr_all;idx++){  
-    if(!regexec(rx,var_lst_all[idx].nm,nbr_sub_xpr,result,flg_exe)){
-      var_xtr_rqs[idx]=True;
+  /* Search format string for matches */
+  if(!regexec(rx,fmt_sng,rx_prn_sub_xpr_nbr,result,flg_exe)) mch_nbr++;
+
+  /* Replace match with desired format */
+  fmt_sng_new=(char *)strdup(fmt_sng);
+  if(mch_nbr){
+    mch_psn_srt=result->rm_so; /* [B] Byte offset from start of string to start of substring */
+    mch_psn_end=result->rm_eo-1L; /* [B] Byte offset from start of string to end   of substring */
+    (void)nco_realloc(fmt_sng_new,(mch_psn_srt+strlen(fmt_sng)-mch_psn_end+2L)*sizeof(char));
+    (void)sprintf(fmt_sng_new+mch_psn_srt,"%%s");
+    (void)sprintf(fmt_sng_new+mch_psn_srt+2,"%s",fmt_sng+mch_psn_end+1);
+  } /* !mch_nbr */
+  
+  if(dbg_lvl_get() > 3) (void)fprintf(stderr,"%s: DEBUG %s reports that the user-supplied formatting string \"%s\" has %d matches to the regular expression \"%s\", which has %zu parenthesized sub-expressions. The first match, if any, begins at offset %d and ends at offset %d and is %d characters long. The revised format string is \"%s\"\n",prg_nm_get(),fnc_nm,fmt_sng,mch_nbr,rx_sng,rx_prn_sub_xpr_nbr,mch_psn_srt,mch_psn_end,mch_psn_end-mch_psn_srt+1,fmt_sng_new);
+
+  regfree(rx); /* Free regular expression data structure */
+  rx=(regex_t *)nco_free(rx);
+  result=(regmatch_t *)nco_free(result);
+
+  return fmt_sng_new;
+} /* end nco_fmt_sng_printf_subst() */
+
+int /* O [nbr] Number of matches found */
+nco_lst_rx_search /* [fnc] Search for pattern matches in list of objects (groups or variables) */
+(const int obj_nbr_all, /* I [nbr] Size of obj_lst_all and flg_obj_mch */
+ nm_id_sct *obj_lst_all, /* I [sct] All objects in input file (with IDs) */
+ char *rx_sng, /* I [sng] Regular expression pattern */
+ nco_bool *flg_obj_mch) /* O [flg] Match flag */
+{
+  /* Purpose: Set flags indicating whether each list member matches given regular expression
+     NB: This function never writes False into a flag, in only writes True.
+     This is because input flags are not assumed to be stateless */
+
+  int obj_idx;
+  int err_id;
+  int flg_cmp; /* Comparison flags */
+  int flg_exe; /* Execution flages */
+  int mch_nbr=0;
+  
+  regmatch_t *result;
+  regex_t *rx;
+
+  size_t rx_prn_sub_xpr_nbr;
+
+  rx=(regex_t *)nco_malloc(sizeof(regex_t));
+
+  /* Choose RE_SYNTAX_POSIX_EXTENDED regular expression type */
+  flg_cmp=(REG_EXTENDED | REG_NEWLINE);
+  /* Set execution flags */
+  flg_exe=0;
+
+  /* Compile regular expression */
+  if((err_id=regcomp(rx,rx_sng,flg_cmp))){ /* Compile regular expression */
+    char const * rx_err_sng;  
+    /* POSIX regcomp return error codes */
+    switch(err_id){
+    case REG_BADPAT: rx_err_sng="Invalid pattern"; break;  
+    case REG_ECOLLATE: rx_err_sng="Not implemented"; break;
+    case REG_ECTYPE: rx_err_sng="Invalid character class name"; break;
+    case REG_EESCAPE: rx_err_sng="Trailing backslash"; break;
+    case REG_ESUBREG: rx_err_sng="Invalid back reference"; break;
+    case REG_EBRACK: rx_err_sng="Unmatched left bracket"; break;
+    case REG_EPAREN: rx_err_sng="Parenthesis imbalance"; break;
+    case REG_EBRACE: rx_err_sng="Unmatched {"; break;
+    case REG_BADBR: rx_err_sng="Invalid contents of { }"; break;
+    case REG_ERANGE: rx_err_sng="Invalid range end"; break;
+    case REG_ESPACE: rx_err_sng="Ran out of memory"; break;
+    case REG_BADRPT: rx_err_sng="No preceding re for repetition op"; break;
+    default: rx_err_sng="Invalid pattern"; break;  
+    } /* end switch */
+    (void)fprintf(stdout,"%s: ERROR nco_lst_rx_search() error in regular expression \"%s\" %s\n",prg_nm_get(),rx_sng,rx_err_sng); 
+    nco_exit(EXIT_FAILURE);
+  } /* end if err */
+
+  rx_prn_sub_xpr_nbr=rx->re_nsub+1L; /* Number of parenthesized sub-expressions */
+
+  /* Search string */
+  result=(regmatch_t *)nco_malloc(sizeof(regmatch_t)*rx_prn_sub_xpr_nbr);
+
+  /* Check each object string for match to rx */
+  for(obj_idx=0;obj_idx<obj_nbr_all;obj_idx++){  
+    /* NB: Here is where flag would be set to False if input were stateless */
+    if(!regexec(rx,obj_lst_all[obj_idx].nm,rx_prn_sub_xpr_nbr,result,flg_exe)){
+      flg_obj_mch[obj_idx]=True;
       mch_nbr++;
     } /* end if */
   } /* end loop over variables */
@@ -83,8 +179,7 @@ nco_lst_meta_search /* [fnc] Search for pattern matches in var string list */
   result=(regmatch_t *)nco_free(result);
 
   return mch_nbr;
-} /* end nco_lst_meta_search() */
-
+} /* end nco_lst_rx_search() */
 #endif /* !NCO_HAVE_REGEX_FUNCTIONALITY */
 
 void 
@@ -340,14 +435,6 @@ nco_lst_prs_2D /* [fnc] Create list of strings from given string and delimiter *
   for(idx=0;idx<*nbr_lst;idx++)
     if(strlen(sng_lst_out[idx]) == 0) sng_lst_out[idx]=NULL;
 
-  if(dbg_lvl_get() == 5){
-    (void)fprintf(stderr,"nco_lst_prs_2D() reports %d elements in list delimited by \"%s\"\n",*nbr_lst,dlm_sng);
-    for(idx=0;idx<*nbr_lst;idx++) 
-      (void)fprintf(stderr,"sng_lst_out[%d] = %s\n",idx,(sng_lst_out[idx] == NULL) ? "NULL" : sng_lst_out[idx]);
-    (void)fprintf(stderr,"\n");
-    (void)fflush(stderr);
-  } /* end debug */
-
   /* Free duplicate of sng_in */
   sng_in_cpy=(char *)nco_free(sng_in_cpy); 
 
@@ -422,11 +509,6 @@ nco_lst_prs_sgl_2D /* [fnc] Create list of strings from given string and delimit
 
 } /* end nco_lst_prs_sgl_2D() */
 
-
-
-
-
-
 int /* O [enm] Comparison result [<,=,>] 0 iff val_1 [<,==,>] val_2 */
 nco_cmp_chr /* [fnc] Compare two characters */
 (const void * val_1, /* I [chr] Character to compare */
@@ -578,7 +660,23 @@ nco_cmp_ptr_unn /* Compare values of two pointer unions of same type */
 } /* end nco_cmp_ptr_unn() */
 
 void 
-nco_lst_comma2hash /* [fnc] Replace commas with hashes when within braces */
+nco_hash2comma /* [fnc] Replace hashes with commas */
+(char * const rx_sng) /* [sng] Regular expression */
+{
+  /* Purpose: Convert hashes in a string to commas
+     Normally this function is called to undo effects of nco_rx_comma2hash() 
+     Thus assumption is that string may be an rx that needs careful handling before evaluation
+     NB: Usually this code may operate on system memory (e.g., optarg) so be very careful 
+     not to overwrite ends of strings */
+  char *sng=rx_sng;
+  while(*sng){
+    if(*sng == '#') *sng=',';
+    sng++;
+  } /* end while */
+} /* end nco_lst_hash2comma() */
+
+void 
+nco_rx_comma2hash /* [fnc] Replace commas with hashes when within braces */
 (char * const rx_sng) /* [sng] Regular expression */
 {
   /* Purpose: Convert commas within braces to hashes within braces in regular expressions
@@ -606,7 +704,7 @@ nco_lst_comma2hash /* [fnc] Replace commas with hashes when within braces */
     /* Increment position in regular expression */
     cp++;
   } /* end while character is not NUL */
-} /* end nco_lst_comma2hash() */
+} /* end nco_rx_comma2hash() */
 
 nm_id_sct * /* O [sct] Sorted output list */
 nco_lst_srt_nm_id /* [fnc] Sort name/ID input list numerically or alphabetically */
@@ -638,11 +736,6 @@ nco_nm_id_lst_free /* [fnc] Free memory associated with name-ID structure list *
 
   for(idx=0;idx<nm_id_nbr;idx++){
     if(nm_id_lst[idx].nm) nm_id_lst[idx].nm=(char *)nco_free(nm_id_lst[idx].nm);
-    /* fxm: not safe to free new nm_id_sct members until code consistently initializes them 
-       this will cause a temporary memory leak until initialization is consistent */
-    /*    if(nm_id_lst[idx].var_nm_fll) nm_id_lst[idx].var_nm_fll=(char *)nco_free(nm_id_lst[idx].var_nm_fll);*/
-    /*    if(nm_id_lst[idx].grp_nm) nm_id_lst[idx].grp_nm=(char *)nco_free(nm_id_lst[idx].grp_nm);*/
-    /*    if(nm_id_lst[idx].grp_nm_fll) nm_id_lst[idx].grp_nm_fll=(char *)nco_free(nm_id_lst[idx].grp_nm_fll);*/
   } /* end loop over idx */
 
   /* Free structure pointer last */
