@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.h,v 1.44 2012/02/14 01:10:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.h,v 1.52 2013/06/17 05:24:33 pvicente Exp $ */
 
 /* Purpose: List utilities */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -19,7 +19,7 @@
 /* Standard header files */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 #include <stdlib.h> /* strtod, strtol, malloc, getopt, qsort */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
 
 #ifdef HAVE_REGEX_H 
 /* 20120213: Linux GCC 4.6 man page says regex.h depends on (non-present) sys/types.h */
@@ -36,6 +36,7 @@
 /* Personal headers */
 #include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_ctl.h" /* Program flow control functions */
+#include "nco_grp_trv.h" /* Group traversal */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_sng_utl.h" /* String utilities */
 
@@ -43,9 +44,13 @@
 extern "C" {
 #endif /* __cplusplus */
 
+char * /* O [nbr] Format string with printf()-formats replaced */
+nco_fmt_sng_printf_subst /* [fnc] Replace printf() format statements */
+(const char * const fmt_sng); /* I [sng] Format string before processing */
+
 int /* O [nbr] Number of matches found */
-nco_lst_meta_search /* [fnc] Search for pattern matches in var string list */
-(int var_nbr_all, /* I [nbr] Size of var_lst_all and var_xtr_rqs */
+nco_lst_rx_search /* [fnc] Search for pattern matches in var string list */
+(const int var_nbr_all, /* I [nbr] Size of var_lst_all and var_xtr_rqs */
  nm_id_sct *var_lst_all, /* I [sct] All variables in input file (with IDs) */
  char *rx_sng, /* I [sng] Regular expression pattern */
  nco_bool *var_xtr_rqs); /* O [flg] Matched vars holder */
@@ -123,7 +128,11 @@ nco_dmn_lst_free /* [fnc] Free memory associated with dimension structure list *
  const int dmn_nbr); /* I [nbr] Number of dimension structures in list */
 
 void 
-nco_lst_comma2hash /* [fnc] Convert {...,...} to {...#...} in regular expressions */
+nco_hash2comma /* [fnc] Replace hashes with commas */
+(char * const rx_sng); /* [sng] Regular expression */
+
+void 
+nco_rx_comma2hash /* [fnc] Convert {...,...} to {...#...} in regular expressions */
 (char * const rx_sng); /* [sng] Regular expression */
 
 nm_id_sct * /* O [sct] Sorted output list */
