@@ -1,64 +1,12 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.c,v 1.146 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.c,v 1.208 2013/09/20 22:46:40 zender Exp $ */
 
 /* Purpose: NCO wrappers for netCDF C library */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
-/* nco_netcdf.h is (nearly) independent of NCO and does not depend on nco.h
-   nco_netcdf.h is an abstraction layer for netcdf.h, plus a few convenience routines
-   A similar abstraction layer must exist for each NCO storage backend, e.g., nco_hdf.h
-
-   1. Utility routines, e.g., nco_typ_lng(), c_typ_nm() (routines with no netCDF library counterpart)
-   2. File-routine wrappers, e.g., nco_open()
-   3. Dimension-routine wrappers, e.g., nco_dimid()
-   4. Variable-routine wrappers, e.g., nco_get_var()
-   5. Attribute-routine wrappers, e.g., nco_put_att()
-
-   Name Convention: Where appropriate, routine name is identical to netCDF C-library name,
-   except nc_ is replaced by nco_
-
-   Argument Ordering Convention: Routines follow same argument order as netCDF C-library
-   Additional arguments, such as nc_type, are appended to end of argument list
-
-   Return value convention: Functions return a success/failure code
-   Errors in netCDF functions cause NCO wrapper to abort, except *_flg() wrappers
-   *_flg() wrappers allow limited, pre-defined, netCDF errors to be non-fatal
-
-   Circularity convention: It is important that none of the error diagnostics
-   in these functions call other nco_??? routines because if everything
-   starts failing then errors will produce circular diagnostics.
-   To ensure this is the case, it is only safe to print diagnostics on
-   variables which are supposed to be valid on input. */
-
-/* NB: nco_netcdf.c does not #include nco.h which #defines forward-compatibility tokens
-   This barrier helps segregate NCO from wrappers
-   Re-define minimal sub-set of tokens for nco_netcdf.c as necessary
-   This is small exception to the barrier */
-#ifndef NC_FORMAT_CLASSIC
-# define NC_FORMAT_CLASSIC (1)
-#endif
-#ifndef NC_FORMAT_64BIT
-# define NC_FORMAT_64BIT   (2)
-#endif
-#ifndef NC_FORMAT_NETCDF4
-# define NC_FORMAT_NETCDF4 (3)
-#endif
-#ifndef NC_FORMAT_NETCDF4_CLASSIC
-# define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
-#endif
-#ifndef NC_CHUNKED
-# define NC_CHUNKED (0)
-#endif
-#ifndef NC_CONTIGUOUS
-# define NC_CONTIGUOUS (1)
-#endif
-#ifndef NC_NOCHECKSUM
-# define NC_NOCHECKSUM 0
-#endif
-
 /* Utility routines not defined by netCDF library, but useful in working with it */
 void
 nco_err_exit /* [fnc] Print netCDF error message, routine name, then exit */
@@ -73,7 +21,7 @@ nco_err_exit /* [fnc] Print netCDF error message, routine name, then exit */
 
      msg variable allows wrapper to pass more descriptive information than
      is contained in the netCDF-defined error message.
-     Use msg to print, e.g., the name of variable or routine which caused error */
+     Use msg to print, e.g., name of variable or routine which caused error */
 
   const char fnc_nm[]="nco_err_exit()";
 #ifdef NCO_ABORT_ON_ERROR
@@ -83,7 +31,19 @@ nco_err_exit /* [fnc] Print netCDF error message, routine name, then exit */
 #endif /* !NCO_ABORT_ON_ERROR */
 
   switch(rcd){
-  case NC_ENOTNC: (void)fprintf(stdout,"ERROR NC_ENOTNC Not a netCDF file\nHINT: NC_ENOTNC errors will occur when NCO operators linked to the netCDF3 library attempt to read netCDF4 files.  Are your input files netCDF4 format?  (http://nco.sf.net/nco.html#fmt_inq shows how to tell.) If so then installing or re-building a netCDF4-compatible version of NCO should solve this problem. First upgrade netCDF to version 4.x, then install NCO using those netCDF 4.x libraries.\n"); break;
+  case NC_EINVAL: (void)fprintf(stdout,"ERROR NC_EINVAL Invalid argument\nHINT: NC_EINVAL errors will occur when NCO operators attempt to open netCDF4 files using the diskless option, usually invoked with --diskless_all, --ram_all, or --open_ram.  Is your input file netCDF4 format?  (http://nco.sf.net/nco.html#fmt_inq shows how to tell.) If so then omitting the diskless option may solve this problem.\n"); break;
+  case NC_ELATEFILL: /* netcdf.h replaced NC_EFILLVALUE by NC_ELATEFILL after about netCDF ~4.2.1 */
+    (void)fprintf(stdout,"ERROR NC_ELATEFILL (formerly NC_EFILLVALUE) Attempt to define fill value when data already exists\nHINT: NC_ELATEFILL errors can occur when ncap2 attempts to define a variable with a _FillValue attribute in a netCDF4 file.  We believe this is an NCO bug (fxm TODO nco1089) and are working to fix it. Does your output file need to be netCDF4 or netCDF4_classic format? If so, then wait for us to fix the bug. If not, change the output format to netCDF3 (e.g., with -3 option) as a temporary workaround. This file can then successfully be converted to netCDF4 (e.g., with ncks -4 in.nc out.nc).\n"); break;
+#ifdef ENABLE_NETCDF4
+  case NC_ENOTBUILT: (void)fprintf(stdout,"ERROR NC_ENOTBUILT Attempt to use feature that was not turned on when netCDF was built\nHINT: NC_ENOTBUILT errors occur only, in our experience, when NCO attempts to access an HDF-EOS2 or HDF4 file. In this case it may be possible to access the input files using NCO if NCO is first re-linked to a version of netCDF configured with the --enable-hdf4 option. This is a non-standard netCDF build option.\n");
+#endif
+  case NC_ENOTNC: (void)fprintf(stdout,"ERROR NC_ENOTNC Not a netCDF file\nHINT: NC_ENOTNC errors can occur for many reasons. If your use case matches one of the four listed below, take the corrective action indicated:\n1. An NCO operator linked to the netCDF3 library attempts to read netCDF4 files. ");
+#ifdef ENABLE_NETCDF4
+    (void)fprintf(stdout," However, this executable seems to have been built with the capability to manipulate netCDF4 files, so it is unlikely that this command failed only because the input datasets are netCDF4 format. Something else is going wrong. \n");
+#else /* !ENABLE_NETCDF4 */
+    (void)fprintf(stdout,"Are your input files netCDF4 format?  (http://nco.sf.net/nco.html#fmt_inq shows how to tell.) If so then installing or re-building a netCDF4-compatible version of NCO should solve this problem. First upgrade netCDF to version 4.x, then install NCO using those netCDF 4.x libraries.\n2. NC_ENOTNC can occur when users attempt to utilize diskless (i.e., RAM) files.  In this case remove the diskless switches (e.g., --ram or --diskless) and then re-issue the command. \n");
+#endif /* !ENABLE_NETCDF4 */
+    (void)fprintf(stdout,"2. NCO attempts to utilize diskless (i.e., RAM) files.  In this case remove the diskless switches (e.g., --ram or --diskless) and then re-issue the command.\n3. NCO attempts to read other non-supported filetypes, e.g., HDF4, HDF-EOS2. In this case it may be possible to access the input files using NCO if NCO is first re-linked to a version of netCDF configured with the --enable-hdf4 option. This is a non-standard netCDF build option.\n4. Access to a DAP URL fails, and the backup method of downloading the URL using wget obtains a data aggregation file (e.g., a .ncml file) instead of an actual netCDF file. In this case the problem is with the DAP server or URL.\n"); break;
   case NC_ERANGE: (void)fprintf(stdout,"ERROR NC_ERANGE Result not representable in output file\nHINT: NC_ERANGE errors typically occur after an arithmetic operation results in a value not representible by the output variable type when NCO attempts to write those values to an output file.  Possible workaround: Promote the variable to higher precision before attempting arithmetic.  For example,\nncap2 -O -s \'foo=double(foo);\' in.nc in.nc\nFor more details, see http://nco.sf.net/nco.html#typ_cnv\n"); break;
   case NC_EVARSIZE: (void)fprintf(stdout,"ERROR NC_EVARSIZE One or more variable sizes violate format constraints\nHINT: NC_EVARSIZE errors can occur when attempting to aggregate netCDF3 classic files together into outputs that exceed the capacity of the netCDF3 classic file format, e.g., a variable with size in excess of 2^31 bytes. In this case, try altering the output file type to netCDF3 classic with 64-bit offsets (with --64) or to netCDF4 (with -4). For more details, see http://nco.sf.net/nco.html#fl_fmt\n"); break;
   } /* end switch */
@@ -161,6 +121,8 @@ const char * /* O [sng] String describing type */
 nco_typ_sng /* [fnc] Convert netCDF type enum to string */
 (const nc_type type) /* I [enm] netCDF type */
 {
+  if(type >= NC_FIRSTUSERTYPEID) return "User-defined";
+
   switch(type){
   case NC_FLOAT:
     return "NC_FLOAT";
@@ -186,6 +148,14 @@ nco_typ_sng /* [fnc] Convert netCDF type enum to string */
     return "NC_UINT64";
   case NC_STRING:
     return "NC_STRING";
+  case NC_VLEN:
+    return "NC_VLEN";
+  case NC_OPAQUE:
+    return "NC_OPAQUE";
+  case NC_ENUM:
+    return "NC_ENUM";
+  case NC_COMPOUND:
+    return "NC_COMPOUND";
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
@@ -193,6 +163,80 @@ nco_typ_sng /* [fnc] Convert netCDF type enum to string */
   return (char *)NULL;
 } /* end nco_typ_sng() */
 
+const char * /* O [sng] Native CDL type */
+cdl_typ_nm /* [fnc] Return string describing native CDL type */
+(const nc_type type) /* I [enm] netCDF type */
+{
+  /* Purpose: Divine CDL type string from netCDF external type enum */
+  switch(type){
+  case NC_FLOAT:
+    return "float";
+  case NC_DOUBLE:
+    return "double";
+  case NC_INT:
+    return "int";
+  case NC_SHORT:
+    return "short";
+  case NC_CHAR:
+    return "char";
+  case NC_BYTE:
+    return "byte";
+  case NC_UBYTE:
+    return "ubyte";
+  case NC_USHORT:
+    return "ushort";
+  case NC_UINT:
+    return "uint";
+  case NC_INT64:
+    return "int64";
+  case NC_UINT64:
+    return "uint64";
+  case NC_STRING:
+    return "string";
+  default: nco_dfl_case_nc_type_err(); break;
+  } /* end switch */
+
+  /* Some compilers, e.g., SGI cc, need return statement to end non-void functions */
+  return (char *)NULL;
+} /* end cdl_typ_nm() */
+
+const char * /* O [sng] Native CDL type suffix */
+cdl_typ_sfx /* [fnc] Return suffix string for CDL type */
+(const nc_type type) /* I [enm] netCDF type */
+{
+  /* Purpose: Return suffix string for CDL */
+  switch(type){
+  case NC_FLOAT:
+    return ".f";
+  case NC_DOUBLE:
+    return ".";
+  case NC_INT:
+    return "";
+  case NC_SHORT:
+    return "s";
+  case NC_CHAR:
+    return "";
+  case NC_BYTE:
+    return "b";
+  case NC_UBYTE:
+    return "ub";
+  case NC_USHORT:
+    return "us";
+  case NC_UINT:
+    return "u";
+  case NC_INT64:
+    return "ll";
+  case NC_UINT64:
+    return "ull";
+  case NC_STRING:
+    return "";
+  default: nco_dfl_case_nc_type_err(); break;
+  } /* end switch */
+
+  /* Some compilers, e.g., SGI cc, need return statement to end non-void functions */
+  return (char *)NULL;
+} /* end cdl_typ_sfx() */
+
 const char * /* O [sng] Native C type */
 c_typ_nm /* [fnc] Return string describing native C type */
 (const nc_type type) /* O [enm] netCDF type */
@@ -348,7 +392,7 @@ nco_dfl_case_nc_type_err(void) /* [fnc] Print error and exit for illegal switch(
      Placing this in its own routine also has the virtue of saving many lines
      of code since this function is used in many many switch() statements. */
   const char fnc_nm[]="nco_dfl_case_nc_type_err()";
-  (void)fprintf(stdout,"%s: ERROR switch(nctype) statement fell through to default case, which is illegal.\nNot handling the default case causes gcc to emit warnings when compiling NCO with the NETCDF2_ONLY token (because nctype definition is braindead in netCDF2.x). Exiting...\n",fnc_nm);
+  (void)fprintf(stdout,"%s: ERROR switch(nctype) statement fell through to default case, which is illegal, because each type should have a well-defined action. This error may be triggered by using an NCO built with only netCDF3 functionality to examine a netCDF4 dataset that contains a new atomic type (e.g., NC_INT64).\nHINT: Configure/build NCO with --enable-netCDF4. Exiting...\n",fnc_nm);
   nco_err_exit(0,fnc_nm);
 } /* end nco_dfl_case_nc_type_err() */
 
@@ -395,10 +439,27 @@ nco_create(const char * const fl_nm,const int cmode,int * const nc_id)
   const char fnc_nm[]="nco_create()";
   int rcd;
   rcd=nc_create(fl_nm,cmode,nc_id);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,fnc_nm);
+  if(rcd != NC_NOERR){
+    (void)fprintf(stdout,"ERROR: %s unable to create file \"%s\"\n",fnc_nm,fl_nm);
+    nco_err_exit(rcd,fnc_nm);
+  } /* endif */
   return rcd;
 } /* end nco_create */
 
+int
+nco__create(const char * const fl_nm,const int cmode,const size_t sz_ntl,size_t * const bfr_sz_hnt,int * const nc_id)
+{
+  /* Purpose: Wrapper for nc__create() */
+  const char fnc_nm[]="nco__create()";
+  int rcd;
+  rcd=nc__create(fl_nm,cmode,sz_ntl,bfr_sz_hnt,nc_id);
+  if(rcd != NC_NOERR){
+    (void)fprintf(stdout,"ERROR: %s unable to create file \"%s\"\n",fnc_nm,fl_nm);
+    nco_err_exit(rcd,fnc_nm);
+  } /* endif */
+  return rcd;
+} /* end nco__create */
+
 int
 nco_open(const char * const fl_nm,const int mode,int * const nc_id)
 {
@@ -406,10 +467,27 @@ nco_open(const char * const fl_nm,const int mode,int * const nc_id)
   const char fnc_nm[]="nco_open()";
   int rcd;
   rcd=nc_open(fl_nm,mode,nc_id);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,fnc_nm);
+  if(rcd != NC_NOERR){
+    (void)fprintf(stdout,"ERROR: %s unable to open file \"%s\"\n",fnc_nm,fl_nm);
+    nco_err_exit(rcd,fnc_nm);
+  } /* endif */
   return rcd;
 } /* end nco_open */
 
+int
+nco__open(const char * const fl_nm,const int mode,size_t * const bfr_sz_hnt,int * const nc_id)
+{
+  /* Purpose: Wrapper for nc__open() */
+  const char fnc_nm[]="nco__open()";
+  int rcd;
+  rcd=nc__open(fl_nm,mode,bfr_sz_hnt,nc_id);
+  if(rcd != NC_NOERR){
+    (void)fprintf(stdout,"ERROR: %s unable to open file \"%s\"\n",fnc_nm,fl_nm);
+    nco_err_exit(rcd,fnc_nm);
+  } /* endif */
+  return rcd;
+} /* end nco__open */
+
 int
 nco_open_flg(const char * const fl_nm,const int mode,int * const nc_id)
 {
@@ -560,7 +638,7 @@ nco_inq_format(const int nc_id,int * const fl_fmt)
   return rcd;
 } /* end nco_inq_format() */
 
-#ifdef ENABLE_NETCDF4
+#ifdef HAVE_NETCDF4_H
 int
 nco_inq_ncid(const int nc_id,const char * const grp_nm,int * const grp_id)
 {
@@ -570,7 +648,18 @@ nco_inq_ncid(const int nc_id,const char * const grp_nm,int * const grp_id)
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_ncid()");
   return rcd;
 } /* end nco_inq_ncid() */
-#endif /* !ENABLE_NETCDF4 */
+
+int
+nco_inq_ncid_flg(const int nc_id,const char * const grp_nm,int * const grp_id)
+{
+  /* Purpose: Error-tolerant Wrapper for nc_inq_ncid(). Tolerates NC_ENOGRP */
+  int rcd;
+  rcd=nc_inq_ncid(nc_id,grp_nm,grp_id);
+  if(rcd == NC_ENOGRP) return rcd;
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_ncid()");
+  return rcd;
+} /* end nco_inq_ncid_flg() */
+#endif /* !HAVE_NETCDF4_H */
 
 int
 nco_inq_ndims(const int nc_id,int * const dmn_nbr_fl)
@@ -625,7 +714,7 @@ nco_inq_unlimdim_flg(const int nc_id,int * const rec_dmn_id)
 /* End File routines */
 
 /* Begin Group routines (_grp) */
-#ifdef ENABLE_NETCDF4
+#ifdef HAVE_NETCDF4_H
 int nco_def_grp(const int nc_id,const char * const grp_nm,int * const grp_id)
 {
   /* Purpose: Wrapper for nc_def_grp() */
@@ -635,6 +724,47 @@ int nco_def_grp(const int nc_id,const char * const grp_nm,int * const grp_id)
   return rcd;
 } /* end nco_def_grp() */
 
+int nco_def_grp_flg(const int nc_id,const char * const grp_nm,int * const grp_id)
+{
+  /* Purpose: Error-tolerant wrapper for nc_def_grp(). Tolerates NC_ENAMEINUSE (-42) "String match to name in use" */
+  int rcd;
+  rcd=nc_def_grp(nc_id,grp_nm,grp_id);
+  if(rcd == NC_ENAMEINUSE) return rcd;
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_def_grp_flg()");
+  return rcd;
+} /* end nco_def_grp_flg() */
+
+#ifndef NC_HAVE_RENAME_GRP
+int
+nc_rename_grp(int grp_id,const char * const grp_nm)
+{
+  /* Purpose: Pseudo-library function to rename groups.
+     This particular stub routine is only called by netCDF4-enabled code
+     when built against a netCDF library that it too old to have
+     the nc_rename_grp() function.
+     Test by running something like this:
+     ncrename -O -g g1,obama ~/nco/data/in_grp.nc ~/foo.nc */
+  const char fnc_nm[]="nc_rename_grp()";
+  char grp_nm_old[NC_MAX_NAME+1L];
+  int rcd;
+  rcd=NC_NOERR;
+  rcd+=nco_inq_grpname(grp_id,grp_nm_old);
+  (void)fprintf(stdout,"INFO: %s reports attempt to rename group \"%s\" to \"%s\" was foiled because libnetcdf.a does not contain nc_rename_grp(). To obtain this functionality, please rebuild NCO against netCDF library version 4.3.1 (to be released ~201309XX) or later.\nContinuing as though nothing untoward happened...\n",fnc_nm,grp_nm_old,grp_nm);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nc_rename_grp()");
+  return rcd;
+} /* end nc_rename_grp() */
+#endif /* NC_HAVE_RENAME_GRP */
+
+int
+nco_rename_grp(int grp_id,const char * const grp_nm)
+{
+  /* Purpose: Wrapper for nc_rename_grp() */
+  int rcd;
+  rcd=nc_rename_grp(grp_id,grp_nm);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_rename_grp()");
+  return rcd;
+} /* end nco_rename_grp() */
+
 int nco_inq_dimids(const int nc_id,int * const dmn_nbr,int * const dmn_ids,int flg_prn)
 {
   /* Purpose: Wrapper for nc_inq_dimids() */
@@ -680,7 +810,7 @@ int nco_inq_grps(const int nc_id,int * const grp_nbr,int * const grp_ids)
   return rcd;
 } /* end nco_inq_grps() */
 
-int nco_inq_grp_ncid(const int nc_id,char * const grp_nm,int * const grp_id)
+int nco_inq_grp_ncid(const int nc_id,const char * const grp_nm,int * const grp_id)
 {
   /* Purpose: Wrapper for nc_inq_grp_ncid() */
   int rcd;
@@ -689,15 +819,47 @@ int nco_inq_grp_ncid(const int nc_id,char * const grp_nm,int * const grp_id)
   return rcd;
 } /* end nco_inq_grp_ncid() */
 
-int nco_inq_grp_full_ncid(const int nc_id,char * const grp_nm_fll,int * const grp_id)
+int nco_inq_grp_ncid_flg(const int nc_id,const char * const grp_nm,int * const grp_id)
+{
+  /* Purpose: Error-tolerant wrapper for nc_inq_grp_ncid(). Tolerates NC_ENOGRP */
+  int rcd;
+  rcd=nc_inq_grp_ncid(nc_id,grp_nm,grp_id);
+  if(rcd == NC_ENOGRP) return rcd;
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_grp_ncid()");
+  return rcd;
+} /* end nco_inq_grp_ncid() */
+
+int nco_inq_grp_full_ncid(const int nc_id,const char * const grp_nm_fll,int * const grp_id)
 {
   /* Purpose: Wrapper for nc_inq_grp_full_ncid() */
+  int fl_fmt;
   int rcd;
-  rcd=nc_inq_grp_full_ncid(nc_id,grp_nm_fll,grp_id);
+  rcd=nco_inq_format(nc_id,&fl_fmt);
+  if(fl_fmt == NC_FORMAT_NETCDF4 || fl_fmt == NC_FORMAT_NETCDF4_CLASSIC){
+    rcd=nc_inq_grp_full_ncid(nc_id,grp_nm_fll,grp_id);
+  }else{ /* netCDF3 */
+    *grp_id=nc_id;
+  } /* endif */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_grp_full_ncid()");
   return rcd;
 } /* end nco_inq_grp_full_ncid() */
 
+int nco_inq_grp_full_ncid_flg(const int nc_id,const char * const grp_nm_fll,int * const grp_id)
+{
+  /* Purpose: Error-tolerant Wrapper for nc_inq_grp_full_ncid(). Tolerates NC_ENOGRP */
+  int fl_fmt;
+  int rcd;
+  rcd=nco_inq_format(nc_id,&fl_fmt);
+  if(fl_fmt == NC_FORMAT_NETCDF4 || fl_fmt == NC_FORMAT_NETCDF4_CLASSIC){
+    rcd=nc_inq_grp_full_ncid(nc_id,grp_nm_fll,grp_id);
+  }else{ /* netCDF3 */
+    *grp_id=nc_id;
+  } /* endif */
+  if(rcd == NC_ENOGRP) return rcd;
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_grp_full_ncid()");
+  return rcd;
+} /* end nco_inq_grp_full_ncid_flg() */
+
 int nco_inq_grp_parent(const int nc_id,int * const prn_id)
 {
   /* Purpose: Wrapper for nc_inq_grp_parent() */
@@ -707,6 +869,16 @@ int nco_inq_grp_parent(const int nc_id,int * const prn_id)
   return rcd;
 } /* end nco_inq_grp_parent() */
 
+int nco_inq_grp_parent_flg(const int nc_id,int * const prn_id)
+{
+  /* Purpose: Error-tolerant Wrapper for nc_inq_grp_parent_flg(). Tolerates NC_ENOGRP */
+  int rcd;
+  rcd=nc_inq_grp_parent(nc_id,prn_id);
+  if(rcd == NC_ENOGRP) return rcd;
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_grp_parent_flg()");
+  return rcd;
+} /* end nco_inq_grp_parent_flg() */
+
 int nco_inq_varids(const int nc_id,int * const var_nbr,int * const var_ids)
 {
   /* Purpose: Wrapper for nc_inq_varids() */
@@ -715,7 +887,17 @@ int nco_inq_varids(const int nc_id,int * const var_nbr,int * const var_ids)
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_varids()");
   return rcd;
 } /* end nco_inq_varids() */
-#endif /* !ENABLE_NETCDF4 */
+
+int
+nco_inq_unlimdims(const int nc_id,int *nbr_dmn_ult,int *dmn_ids_ult)
+{
+  /* Purpose: Wrapper for nc_inq_unlimdims() */
+  int rcd;
+  rcd=nc_inq_unlimdims(nc_id,nbr_dmn_ult,dmn_ids_ult);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nc_inq_unlimdims()");
+  return rcd;
+} /* end nco_inq_unlimdims() */
+#endif /* !HAVE_NETCDF4_H */
 /* End Group routines */
 
 /* Begin Dimension routines (_dim) */
@@ -1144,7 +1326,7 @@ nco_put_var1(const int nc_id,const int var_id,const long * const srt,const void
     /* Next line produces gcc and g++ warning:
        attention : le transtypage annule des qualificateurs du type pointeur ciblé */
   case NC_STRING: rcd=NCO_PUT_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(const char **)vp); break;
-    /* Next line produces g++ warning:
+      /* Next line produces g++ warning:
        erreur: invalid conversion from ‘char* const* const’ to ‘const char** */
     /* case NC_STRING: rcd=NCO_PUT_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(const nco_string * const)vp); break;*/
     /* Next line produces g++ warning:
@@ -1201,6 +1383,7 @@ nco_put_vara(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_PUT_VARA_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VARA_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VARA_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_uint64 *)vp); break;
+    /* NC_STRING prototype next causes same compiler warnings described in nco_put_var1() above */
   case NC_STRING: rcd=NCO_PUT_VARA_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const char **)vp); break;
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
@@ -1253,6 +1436,7 @@ nco_put_vars(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_PUT_VARS_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd, (const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VARS_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd, (const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VARS_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const nco_uint64 *)vp); break;
+    /* NC_STRING prototype next causes same compiler warnings described in nco_put_var1() above */
   case NC_STRING: rcd=NCO_PUT_VARS_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const char **)vp); break;
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
@@ -1306,6 +1490,7 @@ nco_put_varm(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_PUT_VARM_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VARM_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VARM_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_uint64 *)vp); break;
+    /* NC_STRING prototype next causes same compiler warnings described in nco_put_var1() above */
   case NC_STRING: rcd=NCO_PUT_VARM_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const char **)vp); break;
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
@@ -1461,6 +1646,7 @@ nco_put_att(const int nc_id,const int var_id,const char * const att_nm,const nc_
   case NC_UINT: rcd=NCO_PUT_ATT_UINT(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_ATT_INT64(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_ATT_UINT64(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_uint64 *)vp); break;
+    /* NC_STRING prototype next causes same compiler warnings described in nco_put_var1() above */
   case NC_STRING: rcd=NCO_PUT_ATT_STRING(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const char **)vp); break;
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
@@ -1514,6 +1700,56 @@ int nc_def_var_deflate(const int nc_id,const int var_id,const int shuffle,const
 int nc_inq_var_deflate(const int nc_id,const int var_id,int * const shuffle, int * const deflate,int * const dfl_lvl){if(shuffle) *shuffle=0;if(deflate) *deflate=0;if(dfl_lvl) *dfl_lvl=0;return 1;}
 int nc_inq_var_fletcher32(const int nc_id,const int var_id,int * const chk_typ){if(chk_typ) *chk_typ=NC_NOCHECKSUM;return 1;}
 #endif /* HAVE_NETCDF4_H */
+#if !defined(HAVE_NETCDF4_H) 
+/* Stubs thus present a fake library for manipulating netCDF3 files with the netCDF4 API
+   These are only called when netCDF4 library is unavailable, thus I/O assumed to be netCDF3 */
+int nco_def_grp(const int nc_id,const char * const grp_nm,int * const grp_id){assert(0);return NC_NOERR;}
+int nco_rename_grp(int grp_id,const char * const grp_nm){assert(0);return NC_NOERR;}
+int nco_inq_grpname_full(const int nc_id,size_t * grp_nm_lng,char * const grp_nm_fll){assert(0);return NC_NOERR;}
+int nco_inq_grpname_len(const int nc_id,size_t * const grp_nm_lng){assert(0);return NC_NOERR;}
+int nco_inq_grps(const int nc_id,int * const grp_nbr,int * const grp_ids){if(grp_nbr) *grp_nbr=0;return NC_NOERR;}
+int nco_inq_grp_full_ncid(const int nc_id,const char * const grp_nm_fll,int * const grp_id){*grp_id=nc_id;return NC_NOERR;}
+int nco_inq_grp_ncid(const int nc_id,const char * const grp_nm,int * const grp_id){assert(0);return NC_NOERR;}
+int nco_inq_grp_parent(const int nc_id,int * const prn_id){assert(0);return NC_ENOGRP;}
+int nco_inq_grp_parent_flg(const int nc_id,int * const prn_id){return NC_ENOGRP;}
+int nco_inq_ncid_flg(const int nc_id,const char * const grp_nm,int * const grp_id){assert(0);return NC_NOERR;}
+int nco_inq_grp_ncid_flg(const int nc_id,const char * const grp_nm,int * const grp_id){*grp_id=nc_id;return NC_NOERR;}
+int nco_inq_grpname(const int nc_id,char * const grp_nm){if(grp_nm) strcpy(grp_nm,"/");return NC_NOERR;}
+int nco_inq_grp_full_ncid_flg(const int nc_id,const char * const grp_nm_fll,int * const grp_id){*grp_id=nc_id;return NC_NOERR;}
+int nco_inq_dimids(const int nc_id,int * const dmn_nbr,int * const dmn_ids,int flg_prn){
+  int dmn_idx;
+  int rcd;
+  rcd=nc_inq(nc_id,dmn_nbr,(int *)NULL,(int *)NULL,(int *)NULL);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_dimids()");
+  /* netCDF3 file dimension IDs range 0..N-1 */
+  if(dmn_ids)
+    for(dmn_idx=0;dmn_idx<*dmn_nbr;dmn_idx++) dmn_ids[dmn_idx]=dmn_idx;
+  return NC_NOERR;
+} /* end nco_inq_dimids() */
+int nco_inq_unlimdims(const int nc_id,int *nbr_dmn_ult,int *dmn_ids_ult){
+  int rcd;
+  int unlimid;
+  rcd=nc_inq_unlimdim(nc_id,&unlimid);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_unlimdims()");
+  if(unlimid != -1){
+    *nbr_dmn_ult=1;
+    if(dmn_ids_ult) dmn_ids_ult[0]=unlimid;
+  }else{
+    *nbr_dmn_ult=0;
+  } /* unlimid */
+  return NC_NOERR;
+} /* end nco_inq_unlimdims() */
+int nco_inq_varids(const int nc_id,int * const var_nbr,int * const var_ids){
+  int rcd;
+  /* netCDF3 files have only the root group, with variable IDs 0..N-1 */
+  rcd=nc_inq(nc_id,(int *)NULL,var_nbr,(int *)NULL,(int *)NULL);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_varids()");
+  if(var_ids)
+    for(int var_idx=0;var_idx<*var_nbr;var_idx++) var_ids[var_idx]=var_idx;
+  return NC_NOERR;
+} /* end nco_inq_varids() */
+#endif /* HAVE_NETCDF4_H */
+#ifndef _MSC_VER
 #ifndef ENABLE_NETCDF4
 int NCO_GET_VAR1_UBYTE(const int nc_id,const int var_id,const size_t *srt,nco_ubyte *ubp){return 1;}
 int NCO_GET_VAR1_USHORT(const int nc_id,const int var_id,const size_t *srt,nco_ushort *usp){return 1;}
@@ -1596,4 +1832,5 @@ int NCO_GET_ATT_INT64(const int nc_id,const int var_id,const char *att_nm,nco_in
 int NCO_GET_ATT_UINT64(const int nc_id,const int var_id,const char *att_nm,nco_uint64 *ui64p){return 1;}
 int NCO_GET_ATT_STRING(const int nc_id,const int var_id,const char *att_nm,nco_string *sngp){return 1;}
 #endif /* ENABLE_NETCDF4 */
+#endif /* _MSC_VER */
 /* end netCDF4 stubs */
