@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_pck.c,v 1.89 2012/02/14 05:39:04 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_pck.c,v 1.95 2013/08/01 05:02:07 zender Exp $ */
 
 /* Purpose: NCO utilities for packing and unpacking variables */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -466,7 +466,7 @@ nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
     var->typ_upk=(var->has_scl_fct) ? scl_fct_typ : add_fst_typ; /* [enm] Type of variable when unpacked (expanded) (in memory) */
     if(nco_is_rth_opr(prg_get()) && dbg_lvl_get() >= nco_dbg_var){
       (void)fprintf(stdout,"%s: PACKING Variable %s is type %s packed into type %s\n",prg_nm_get(),var->nm,nco_typ_sng(var->typ_upk),nco_typ_sng(var->typ_dsk));
-      (void)fprintf(stdout,"%s: DEBUG Packed variables processed by all arithmetic operators are unpacked automatically, and then stored unpacked in the output file. If you wish to repack them in the output file, use, e.g., ncap -O -s \"foo=pack(foo);\" out.nc out.nc. If you wish to pack all variables in a file, use, e.g., ncpdq -P all_new in.nc out.nc.\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: DEBUG Packed variables processed by all arithmetic operators are unpacked automatically, and then stored unpacked in the output file. If you wish to repack them in the output file, use, e.g., ncap2 -O -s \"foo=pack(foo);\" out.nc out.nc. If you wish to pack all variables in a file, use, e.g., ncpdq -P all_new in.nc out.nc.\n",prg_nm_get());
     } /* endif print packing information */
   }else{
     /* Variable is not packed since neither scale factor nor add_offset exist
@@ -1069,31 +1069,73 @@ nco_var_upk /* [fnc] Unpack variable in memory */
   } /* endif */
 
   /* Packed variables are not guaranteed to have both scale_factor and add_offset
-     scale_factor is guaranteed to be of type NC_FLOAT or NC_DOUBLE and of size 1 (a scalar) */
+     scale_factor is guaranteed to be of type NC_FLOAT or NC_DOUBLE and of size 1 (a scalar)
+     Hence algorithm create scalar value structures from values of scale_factor, add_offset */
 
-  /* Create scalar value structures from values of scale_factor, add_offset */
-  if(var->has_scl_fct){ /* [flg] Valid scale_factor attribute exists */
-    scv_sct scl_fct_scv;
-    var->scl_fct.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
-    (void)nco_get_att(var->nc_id,var->id,scl_fct_sng,var->scl_fct.vp,var->typ_upk);
-    scl_fct_scv=ptr_unn_2_scv(var->typ_upk,var->scl_fct);
-    /* Convert var to type of scale_factor for expansion */
-    var=nco_var_cnf_typ(scl_fct_scv.type,var);
-    /* Multiply var by scale_factor */
-    (void)var_scv_mlt(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scl_fct_scv);
-  } /* endif has_scl_fct */
+  /* 20130729:
+     Rumors are true: 
+     NASA HDF data, including NASA MODIS swaths, expect offsets to be _subtracted_ then scaled!
+     http://modis-atmos.gsfc.nasa.gov/MOD08_D3/faq.html
+     The netCDF convention for packing is opposite:
+     http://www.unidata.ucar.edu/software/netcdf/docs/netcdf/Attribute-Conventions.html
+     Hence unpacking NASA SDS data requires re-ordering and re-defining the netCDF-standard unpacking algorithm */
 
-  if(var->has_add_fst){ /* [flg] Valid add_offset attribute exists */
-    scv_sct add_fst_scv;
-    var->add_fst.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
-    /* fxm TODO nco638 */
-    (void)nco_get_att(var->nc_id,var->id,add_fst_sng,var->add_fst.vp,var->typ_upk);
-    add_fst_scv=ptr_unn_2_scv(var->typ_upk,var->add_fst);
-    /* Convert var to type of scale_factor for expansion */
-    var=nco_var_cnf_typ(add_fst_scv.type,var);
-    /* Add add_offset to var */
-    (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&add_fst_scv);
-  } /* endif has_add_fst */
+  /* Test unpacking conventions:
+     ncpdq -O -C -U       -v ^pck_.? ~/nco/data/in.nc ~/foo.nc # Unpack netCDF
+     ncpdq -O -C -U --hdf -v ^pck_.? ~/nco/data/in.nc ~/foo.nc # Unpack HDF
+     ncpdq -O -C -P --hdf -v ^pck_.? ~/nco/data/in.nc ~/foo.nc # Unpack HDF and re-pack netCDF */
+
+  if(nco_upk_cnv_get() == nco_upk_netCDF){
+    /* netCDF unpack definition: unpacked=(scale_factor*packed)+add_offset */
+    
+    if(var->has_scl_fct){ /* [flg] Valid scale_factor attribute exists */
+      scv_sct scl_fct_scv;
+      var->scl_fct.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
+      (void)nco_get_att(var->nc_id,var->id,scl_fct_sng,var->scl_fct.vp,var->typ_upk);
+      scl_fct_scv=ptr_unn_2_scv(var->typ_upk,var->scl_fct);
+      /* Convert var to type of scale_factor for expansion */
+      var=nco_var_cnf_typ(scl_fct_scv.type,var);
+      /* Multiply var by scale_factor */
+      (void)var_scv_mlt(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scl_fct_scv);
+    } /* endif has_scl_fct */
+    
+    if(var->has_add_fst){ /* [flg] Valid add_offset attribute exists */
+      scv_sct add_fst_scv;
+      var->add_fst.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
+      (void)nco_get_att(var->nc_id,var->id,add_fst_sng,var->add_fst.vp,var->typ_upk);
+      add_fst_scv=ptr_unn_2_scv(var->typ_upk,var->add_fst);
+      /* Convert var to type of scale_factor for expansion */
+      var=nco_var_cnf_typ(add_fst_scv.type,var);
+      /* Add add_offset to var */
+      (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&add_fst_scv);
+    } /* endif has_add_fst */
+
+  }else{ /* !netCDF_unpack_convention */
+    /* 20130729: NASA HDF unpack definition: unpacked=scale_factor*(packed-add_offset) */
+
+    if(var->has_add_fst){ /* [flg] Valid add_offset attribute exists */
+      scv_sct add_fst_scv;
+      var->add_fst.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
+      (void)nco_get_att(var->nc_id,var->id,add_fst_sng,var->add_fst.vp,var->typ_upk);
+      add_fst_scv=ptr_unn_2_scv(var->typ_upk,var->add_fst);
+      /* Convert var to type of scale_factor for expansion */
+      var=nco_var_cnf_typ(add_fst_scv.type,var);
+      /* Subtract add_offset from var */
+      (void)var_scv_sub(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&add_fst_scv);
+    } /* endif has_add_fst */
+
+    if(var->has_scl_fct){ /* [flg] Valid scale_factor attribute exists */
+      scv_sct scl_fct_scv;
+      var->scl_fct.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
+      (void)nco_get_att(var->nc_id,var->id,scl_fct_sng,var->scl_fct.vp,var->typ_upk);
+      scl_fct_scv=ptr_unn_2_scv(var->typ_upk,var->scl_fct);
+      /* Convert var to type of scale_factor for expansion */
+      var=nco_var_cnf_typ(scl_fct_scv.type,var);
+      /* Multiply var by scale_factor */
+      (void)var_scv_mlt(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scl_fct_scv);
+    } /* endif has_scl_fct */
+    
+  } /* !netCDF_unpack_convention */
 
   if(var->has_mss_val) var=nco_cnv_mss_val_typ(var,var->type);
 
@@ -1173,4 +1215,3 @@ nco_var_upk_swp /* [fnc] Unpack var_in into var_out */
   var_tmp->val.vp=NULL;
   if(var_tmp) var_tmp=nco_var_free(var_tmp);
 } /* end nco_var_upk_swp() */
-
