@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.129 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.157 2013/09/13 18:01:22 zender Exp $ */
 
 /* ncrename -- netCDF renaming operator */
 
 /* Purpose: Rename dimensions, variables, and attributes of a netCDF file */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
 
    License: GNU General Public License (GPL) Version 3
    The full license text is at http://www.gnu.org/copyleft/gpl.html 
@@ -30,9 +30,10 @@
    Irvine, CA 92697-3100 */
 
 /* Usage:
-   ncrename -d old_dim1,new_dim1 -v old_var1,new_var1 -v old_var2,new_var2 -a old_att1,new_att1 in.nc ~/foo.nc
-   ncrename -d lon,new_lon -v scalar_var,new_scalar_var -a long_name,new_long_name in.nc ~/foo.nc
-   */
+   ncrename -O -g g1,obama ~/nco/data/in_grp.nc ~/foo.nc
+   ncrename -O -a Conventions,Geneva ~/nco/data/in.nc ~/foo.nc
+   ncrename -O -d old_dim1,new_dim1 -v old_var1,new_var1 -v old_var2,new_var2 -a old_att1,new_att1 ~/nco/data/in.nc ~/foo.nc
+   ncrename -O -d lon,new_lon -v scalar_var,new_scalar_var -a long_name,new_long_name ~/nco/data/in.nc ~/foo.nc */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h> /* Autotools tokens */
@@ -42,10 +43,12 @@
 #include <math.h> /* sin cos cos sin 3.14159 */
 #include <stdio.h> /* stderr, FILE, NULL, etc. */
 #include <stdlib.h> /* atof, atoi, malloc, getopt */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
-#include <unistd.h> /* all sorts of POSIX stuff */
+#ifndef _MSC_VER
+# include <unistd.h> /* POSIX stuff */
+#endif
 #ifndef HAVE_GETOPT_LONG
 # include "nco_getopt.h"
 #else /* HAVE_GETOPT_LONG */ 
@@ -54,9 +57,8 @@
 # endif /* !HAVE_GETOPT_H */ 
 #endif /* HAVE_GETOPT_LONG */
 
-/* 3rd party vendors */
-#include <netcdf.h> /* netCDF definitions and C library */
-#include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
+/* 3rd party vendors */	 
+#include <netcdf.h> /* netCDF definitions and C library */	 
 
 /* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
 #define MAIN_PROGRAM_FILE
@@ -65,13 +67,19 @@
 int 
 main(int argc,char **argv)
 {
-  nco_bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+#ifndef __GNUG__
+  extern int errno; /* [enm] Error code in errno.h */
+#endif /* __GNUG__ */
+
+  nco_bool FL_RTR_RMT_LCN;
   nco_bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
   nco_bool FORCE_APPEND=False; /* Option A */
   nco_bool FORCE_OVERWRITE=False; /* Option O */
   nco_bool HISTORY_APPEND=True; /* Option h */
-  nco_bool OUTPUT_TO_NEW_NETCDF_FILE=False;
-  nco_bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
+  nco_bool IS_GLB_GRP_ATT=False; /* [flg] Attribute is Global or Group attribute */
+  nco_bool FL_OUT_NEW=False;
+  nco_bool RAM_OPEN=False; /* [flg] Open (netCDF3-only) file(s) in RAM */
+  nco_bool RM_RMT_FL_PST_PRC=True; /* Option R */
   nco_bool flg_cln=False; /* [flg] Clean memory prior to exit */
   
   char **fl_lst_abb=NULL; /* Option a */
@@ -83,14 +91,18 @@ main(int argc,char **argv)
   char *fl_out=NULL; /* Option o */
   char *fl_pth=NULL; /* Option p */
   char *fl_pth_lcl=NULL; /* Option l */
+  char *grp_rnm_arg[NC_MAX_DIMS];
   char *opt_crr=NULL; /* [sng] String representation of current long-option name */
-  char *sng_cnv_rcd=char_CEWI; /* [sng] strtol()/strtoul() return code */
+  char *sng_cnv_rcd=NULL_CEWI; /* [sng] strtol()/strtoul() return code */
   char *var_rnm_arg[NC_MAX_VARS];
 
-  const char * const CVS_Id="$Id: ncrename.c,v 1.129 2012/01/01 20:51:53 zender Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.129 $";
-  const char * const opt_sht_lst="a:D:d:hl:Oo:p:rv:-:";
+  char var_nm[NC_MAX_NAME+1];
 
+  const char * const CVS_Id="$Id: ncrename.c,v 1.157 2013/09/13 18:01:22 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.157 $";
+  const char * const opt_sht_lst="a:D:d:g:hl:Oo:p:rv:-:";
+  const char dlm_chr='@'; /* Character delimiting variable from attribute name  */
+  const char opt_chr='.'; /* Character indicating presence of following variable/dimension/attribute in file is optional */
 #if defined(__cplusplus) || defined(PGI_CC)
   ddra_info_sct ddra_info;
   ddra_info.flg_ddra=False;
@@ -104,8 +116,10 @@ main(int argc,char **argv)
   int abb_arg_nbr=0;
   int fl_nbr=0;
   int idx;
+  int md_open; /* [enm] Mode flag for nc_open() call */
   int nbr_att_rnm=0; /* Option a. NB: nbr_att_rnm gets incremented */
   int nbr_dmn_rnm=0; /* Option d. NB: nbr_dmn_rnm gets incremented */
+  int nbr_grp_rnm=0; /* Option g. NB: nbr_grp_rnm gets incremented */
   int nbr_var_rnm=0; /* Option v. NB: nbr_var_rnm gets incremented */
   int nc_id;  
   int opt;
@@ -113,8 +127,10 @@ main(int argc,char **argv)
   
   rnm_sct *var_rnm_lst=NULL_CEWI;
   rnm_sct *dmn_rnm_lst=NULL_CEWI;
+  rnm_sct *grp_rnm_lst=NULL_CEWI;
   rnm_sct *att_rnm_lst=NULL_CEWI;
 
+  size_t bfr_sz_hnt=NC_SIZEHINT_DEFAULT; /* [B] Buffer size hint */
   size_t hdr_pad=0UL; /* [B] Pad at end of header section */
 
   static struct option opt_lng[] =
@@ -126,9 +142,15 @@ main(int argc,char **argv)
       {"drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"dirty",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"mmr_drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      {"ram_all",no_argument,0,0}, /* [flg] Open (netCDF3) and create file(s) in RAM */
+      {"create_ram",no_argument,0,0}, /* [flg] Create file in RAM */
+      {"open_ram",no_argument,0,0}, /* [flg] Open (netCDF3) file(s) in RAM */
+      {"diskless_all",no_argument,0,0}, /* [flg] Open (netCDF3) and create file(s) in RAM */
       {"version",no_argument,0,0},
       {"vrs",no_argument,0,0},
       /* Long options with argument, no short option counterpart */
+      {"bfr_sz_hnt",required_argument,0,0}, /* [B] Buffer size hint */
+      {"buffer_size_hint",required_argument,0,0}, /* [B] Buffer size hint */
       {"hdr_pad",required_argument,0,0},
       {"header_pad",required_argument,0,0},
       /* Long options with short counterparts */
@@ -137,6 +159,8 @@ main(int argc,char **argv)
       {"dbg_lvl",required_argument,0,'D'},
       {"dimension",required_argument,0,'d'},
       {"dmn",required_argument,0,'d'},
+      {"group",required_argument,0,'g'},
+      {"grp",required_argument,0,'g'},
       {"history",no_argument,0,'h'},
       {"hst",no_argument,0,'h'},
       {"local",required_argument,0,'l'},
@@ -149,6 +173,7 @@ main(int argc,char **argv)
       {"revision",no_argument,0,'r'},
       {"variable",required_argument,0,'v'},
       {"help",no_argument,0,'?'},
+      {"hlp",no_argument,0,'?'},
       {0,0,0,0}
     };
   int opt_idx=0;  /* Index of current long option into opt_lng array */
@@ -172,12 +197,17 @@ main(int argc,char **argv)
 
     /* Process long options without short option counterparts */
     if(opt == 0){
+      if(!strcmp(opt_crr,"bfr_sz_hnt") || !strcmp(opt_crr,"buffer_size_hint")){
+	bfr_sz_hnt=strtoul(optarg,&sng_cnv_rcd,NCO_SNG_CNV_BASE10);
+	if(*sng_cnv_rcd) nco_sng_cnv_err(optarg,"strtoul",sng_cnv_rcd);
+      } /* endif cnk */
       if(!strcmp(opt_crr,"cln") || !strcmp(opt_crr,"mmr_cln") || !strcmp(opt_crr,"clean")) flg_cln=True; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"drt") || !strcmp(opt_crr,"mmr_drt") || !strcmp(opt_crr,"dirty")) flg_cln=False; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"hdr_pad") || !strcmp(opt_crr,"header_pad")){
 	hdr_pad=strtoul(optarg,&sng_cnv_rcd,NCO_SNG_CNV_BASE10);
 	if(*sng_cnv_rcd) nco_sng_cnv_err(optarg,"strtoul",sng_cnv_rcd);
       } /* endif "hdr_pad" */
+      if(!strcmp(opt_crr,"ram_all") || !strcmp(opt_crr,"open_ram") || !strcmp(opt_crr,"diskless_all")) RAM_OPEN=True; /* [flg] Create file in RAM */
       if(!strcmp(opt_crr,"vrs") || !strcmp(opt_crr,"version")){
 	(void)nco_vrs_prn(CVS_Id,CVS_Revision);
 	nco_exit(EXIT_SUCCESS);
@@ -198,10 +228,14 @@ main(int argc,char **argv)
       dbg_lvl=(unsigned short int)strtoul(optarg,&sng_cnv_rcd,NCO_SNG_CNV_BASE10);
       if(*sng_cnv_rcd) nco_sng_cnv_err(optarg,"strtoul",sng_cnv_rcd);
       break;
-    case 'd': /* Copy limit argument for later processing */
+    case 'd': /* Copy argument for later processing */
       dmn_rnm_arg[nbr_dmn_rnm]=(char *)strdup(optarg);
       nbr_dmn_rnm++;
       break;
+    case 'g': /* Copy argument for later processing */
+      grp_rnm_arg[nbr_grp_rnm]=(char *)strdup(optarg);
+      nbr_grp_rnm++;
+      break;
     case 'h': /* Toggle appending to history global attribute */
       HISTORY_APPEND=!HISTORY_APPEND;
       break;
@@ -218,7 +252,7 @@ main(int argc,char **argv)
       fl_pth=(char *)strdup(optarg);
       break;
     case 'R': /* Toggle removal of remotely-retrieved-files. Default is True. */
-      REMOVE_REMOTE_FILES_AFTER_PROCESSING=!REMOVE_REMOTE_FILES_AFTER_PROCESSING;
+      RM_RMT_FL_PST_PRC=!RM_RMT_FL_PST_PRC;
       break;
     case 'r': /* Print CVS program information and copyright notice */
       (void)nco_vrs_prn(CVS_Id,CVS_Revision);
@@ -240,6 +274,7 @@ main(int argc,char **argv)
       nco_exit(EXIT_FAILURE);
       break;
     default: /* Print proper usage */
+      (void)fprintf(stdout,"%s ERROR in command-line syntax/options. Please reformulate command accordingly.\n",prg_nm_get());
       (void)nco_usg_prn();
       nco_exit(EXIT_FAILURE);
     } /* end switch */
@@ -248,69 +283,43 @@ main(int argc,char **argv)
   
   /* Process positional arguments and fill in filenames */
   fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
-  if(fl_out) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=(char *)strdup(fl_lst_in[0]);
+  if(fl_out) FL_OUT_NEW=True; else fl_out=(char *)strdup(fl_lst_in[0]);
 
-  if(nbr_var_rnm == 0 && nbr_att_rnm == 0 && nbr_dmn_rnm == 0){
+  if(!nbr_var_rnm && !nbr_att_rnm && !nbr_grp_rnm && !nbr_dmn_rnm){
     (void)fprintf(stdout,"%s: ERROR must specify something to rename\n",prg_nm);
     nco_usg_prn();
     nco_exit(EXIT_FAILURE);
   } /* end if */ 
 
   /* Make uniform list of user-specified rename structures */
-  if(nbr_var_rnm > 0) var_rnm_lst=nco_prs_rnm_lst(nbr_var_rnm,var_rnm_arg);
-  if(nbr_dmn_rnm > 0) dmn_rnm_lst=nco_prs_rnm_lst(nbr_dmn_rnm,dmn_rnm_arg);
   if(nbr_att_rnm > 0) att_rnm_lst=nco_prs_rnm_lst(nbr_att_rnm,att_rnm_arg);
+  if(nbr_dmn_rnm > 0) dmn_rnm_lst=nco_prs_rnm_lst(nbr_dmn_rnm,dmn_rnm_arg);
+  if(nbr_grp_rnm > 0) grp_rnm_lst=nco_prs_rnm_lst(nbr_grp_rnm,grp_rnm_arg);
+  if(nbr_var_rnm > 0) var_rnm_lst=nco_prs_rnm_lst(nbr_var_rnm,var_rnm_arg);
 
   /* We have final list of variables, dimensions, and attributes to rename */
   
   /* Parse filename */
   fl_in=nco_fl_nm_prs(fl_in,0,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
-  fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
+  fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FL_RTR_RMT_LCN);
 
-  if(OUTPUT_TO_NEW_NETCDF_FILE){
-
-    if(!FORCE_OVERWRITE){
-      int rcd_lcl;
-
-      struct stat stat_sct;
-      
-      rcd_lcl=stat(fl_out,&stat_sct);
-
-      /* If file already exists, then query user whether to overwrite */
-      if(rcd_lcl != -1){
-        char usr_reply='z';
-	short nbr_itr=0;
-
-        while(usr_reply != 'n' && usr_reply != 'y'){
-	  nbr_itr++;
-	  if(nbr_itr > 10){
-	    (void)fprintf(stdout,"\n%s: ERROR %d failed attempts to obtain valid interactive input. Assuming non-interactive shell and exiting.\n",prg_nm_get(),nbr_itr-1);
-	    nco_exit(EXIT_FAILURE);
-	  } /* end if */
-          (void)fprintf(stdout,"%s: overwrite %s (y/n)? ",prg_nm,fl_out);
-	  (void)fflush(stdout);
-          usr_reply=(char)fgetc(stdin);
-	  /* Allow one carriage return per response free of charge */
-	  if(usr_reply == '\n') usr_reply=(char)fgetc(stdin);
-	  (void)fflush(stdin);
-        } /* end while */
-        
-        if(usr_reply == 'n'){
-          nco_exit(EXIT_SUCCESS);
-        } /* end if */
-      } /* end if rcd_lcl != -1 */
-    } /* end if FORCE_OVERWRITE */
+  if(FL_OUT_NEW){
+    /* Obtain user consent, if needed, to overwrite output file (or die trying) */
+    if(!FORCE_OVERWRITE) nco_fl_overwrite_prm(fl_out);
     
     /* Copy input file to output file and then search through output, 
        changing names on the fly. This avoids possible XDR translation
        performance penalty of copying each variable with netCDF. */
     (void)nco_fl_cp(fl_in,fl_out);
 
-  } /* end if OUTPUT_TO_NEW_NETCDF_FILE */
+    /* Ensure output file is user/owner-writable */
+    (void)nco_fl_chmod(fl_out);
+  } /* end if FL_OUT_NEW */
   
   /* Open file enabled for writing. Place file in define mode for renaming. */
-  rcd=nco_open(fl_out,NC_WRITE,&nc_id);
+  if(RAM_OPEN) md_open=NC_WRITE|NC_DISKLESS; else md_open=NC_WRITE;
+  rcd+=nco_fl_open(fl_out,md_open,&bfr_sz_hnt,&nc_id);
   (void)nco_redef(nc_id);
 
   /* Timestamp end of metadata setup and disk layout */
@@ -319,32 +328,50 @@ main(int argc,char **argv)
 
   /* Without further ado, change names */
   for(idx=0;idx<nbr_var_rnm;idx++){
-    if(var_rnm_lst[idx].old_nm[0] == '.'){
-      rcd=nco_inq_varid_flg(nc_id,var_rnm_lst[idx].old_nm+1,&var_rnm_lst[idx].id);
+    if(var_rnm_lst[idx].old_nm[0] == opt_chr){
+      rcd=nco_inq_varid_flg(nc_id,var_rnm_lst[idx].old_nm+1L,&var_rnm_lst[idx].id);
       if(rcd == NC_NOERR){
 	(void)nco_rename_var(nc_id,var_rnm_lst[idx].id,var_rnm_lst[idx].new_nm);
-	if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed variable \'%s\' to \'%s\'\n",var_rnm_lst[idx].old_nm+1,var_rnm_lst[idx].new_nm);
+	if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed variable \'%s\' to \'%s\'\n",prg_nm,var_rnm_lst[idx].old_nm+1L,var_rnm_lst[idx].new_nm);
       }else{
-	(void)fprintf(stderr,"%s: WARNING Variable \"%s\" not present in %s, skipping it.\n",prg_nm,var_rnm_lst[idx].old_nm+1,fl_in);
+	(void)fprintf(stderr,"%s: WARNING Variable \"%s\" not present in %s, skipping it.\n",prg_nm,var_rnm_lst[idx].old_nm+1L,fl_in);
 	/* Reset error code */
 	rcd=NC_NOERR; 
       } /* end if */
-    }else{ /* Variable name does not contain '.' so variable presence is required */
+    }else{ /* Variable name does not contain opt_chr so variable presence is required */
       rcd=nco_inq_varid(nc_id,var_rnm_lst[idx].old_nm,&var_rnm_lst[idx].id);
       (void)nco_rename_var(nc_id,var_rnm_lst[idx].id,var_rnm_lst[idx].new_nm);
-      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed variable \'%s\' to \'%s\'\n",var_rnm_lst[idx].old_nm,var_rnm_lst[idx].new_nm);
+      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed variable \'%s\' to \'%s\'\n",prg_nm,var_rnm_lst[idx].old_nm,var_rnm_lst[idx].new_nm);
+    } /* end else */
+  } /* end loop over idx */
+
+  for(idx=0;idx<nbr_grp_rnm;idx++){
+    if(grp_rnm_lst[idx].old_nm[0] == opt_chr){
+      rcd=nco_inq_grp_ncid_flg(nc_id,grp_rnm_lst[idx].old_nm+1L,&grp_rnm_lst[idx].id);
+      if(rcd == NC_NOERR){
+	(void)nco_rename_grp(grp_rnm_lst[idx].id,grp_rnm_lst[idx].new_nm);
+	if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed group \'%s\' to \'%s\'\n",prg_nm,grp_rnm_lst[idx].old_nm+1L,grp_rnm_lst[idx].new_nm);
+      }else{
+	(void)fprintf(stderr,"%s: WARNING Group \"%s\" not present in %s, skipping it.\n",prg_nm,grp_rnm_lst[idx].old_nm+1L,fl_in);
+	/* Reset error code */
+	rcd=NC_NOERR; 
+      } /* end if */
+    }else{ /* Group name does not contain opt_chr so group presence is required */
+      rcd=nco_inq_grp_ncid(nc_id,grp_rnm_lst[idx].old_nm,&grp_rnm_lst[idx].id);
+      (void)nco_rename_grp(grp_rnm_lst[idx].id,grp_rnm_lst[idx].new_nm);
+      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed group \'%s\' to \'%s\'\n",prg_nm,grp_rnm_lst[idx].old_nm,grp_rnm_lst[idx].new_nm);
     } /* end else */
   } /* end loop over idx */
 
   for(idx=0;idx<nbr_dmn_rnm;idx++){
-    if(dmn_rnm_lst[idx].old_nm[0] == '.'){
+    if(dmn_rnm_lst[idx].old_nm[0] == opt_chr){
       
-      rcd=nco_inq_dimid_flg(nc_id,dmn_rnm_lst[idx].old_nm+1,&dmn_rnm_lst[idx].id);
+      rcd=nco_inq_dimid_flg(nc_id,dmn_rnm_lst[idx].old_nm+1L,&dmn_rnm_lst[idx].id);
       if(rcd == NC_NOERR){
 	(void)nco_rename_dim(nc_id,dmn_rnm_lst[idx].id,dmn_rnm_lst[idx].new_nm);
-	if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed dimension \'%s\' to \'%s\'\n",dmn_rnm_lst[idx].old_nm+1,dmn_rnm_lst[idx].new_nm);
+	if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed dimension \'%s\' to \'%s\'\n",prg_nm,dmn_rnm_lst[idx].old_nm+1L,dmn_rnm_lst[idx].new_nm);
       }else{
-	(void)fprintf(stderr,"%s: WARNING Dimension \'%s\' not present in %s, skipping it.\n",prg_nm,dmn_rnm_lst[idx].old_nm+1,fl_in);
+	(void)fprintf(stderr,"%s: WARNING Dimension \'%s\' not present in %s, skipping it.\n",prg_nm,dmn_rnm_lst[idx].old_nm+1L,fl_in);
 	/* Reset error code */
 	rcd=NC_NOERR; 
       } /* end if */
@@ -352,7 +379,7 @@ main(int argc,char **argv)
     }else{
       rcd=nco_inq_dimid(nc_id,dmn_rnm_lst[idx].old_nm,&dmn_rnm_lst[idx].id);
       (void)nco_rename_dim(nc_id,dmn_rnm_lst[idx].id,dmn_rnm_lst[idx].new_nm);
-      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed dimension \'%s\' to \'%s\'\n",dmn_rnm_lst[idx].old_nm,dmn_rnm_lst[idx].new_nm);
+      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed dimension \'%s\' to \'%s\'\n",prg_nm,dmn_rnm_lst[idx].old_nm,dmn_rnm_lst[idx].new_nm);
     } /* end else */
   } /* end loop over idx */
 
@@ -365,99 +392,115 @@ main(int argc,char **argv)
     for(idx=0;idx<nbr_att_rnm;idx++){
       int var_id;
       int nbr_rnm=0;
+      int rcd_att=0;
       
       /* Rename attribute of single variable... */
-      if(strchr(att_rnm_lst[idx].old_nm,'@')){
-	/* Get variable name from old name */
-	char var_nm[NC_MAX_NAME];
-	if(nco_prs_att((att_rnm_lst+idx),var_nm)){
-	  /* Get var_id of variable */
-	  if(var_nm[0] == '.'){
-	    rcd=nco_inq_varid_flg(nc_id,var_nm+1,&var_id);
+      if(strchr(att_rnm_lst[idx].old_nm,dlm_chr)){
+	/* Extract variable name from old name */
+	rcd_att=nco_prs_att((att_rnm_lst+idx),var_nm,&IS_GLB_GRP_ATT);
+	if(!rcd_att){
+	  (void)fprintf(stderr,"%s: ERROR Could not parse var_nm@att_nm string \"%s\"\n",prg_nm,att_rnm_lst[idx].old_nm);
+	  nco_exit(EXIT_FAILURE);
+	} /* end if */ 
+	/* Get var_id of variable */
+	if(IS_GLB_GRP_ATT){
+	  (void)fprintf(stderr,"%s: INFO Assuming \"%s\" refers to a Global or Group attribute\n",prg_nm,att_rnm_lst[idx].old_nm);
+	  var_id=NC_GLOBAL;
+	}else{ /* !IS_GLB_GRP_ATT */
+	  if(var_nm[0] == opt_chr){
+	    rcd=nco_inq_varid_flg(nc_id,var_nm+1L,&var_id);
 	    if(rcd != NC_NOERR){
-	      (void)fprintf(stderr,"%s: WARNING Variable \'%s\' not present in %s, skipping it.\n",prg_nm,var_nm+1,fl_in);
+	      (void)fprintf(stderr,"%s: WARNING Variable \'%s\' not present in %s, skipping it.\n",prg_nm,var_nm+1L,fl_in);
 	      /* Reset error code */
 	      rcd=NC_NOERR; 
+	      /* Optional variable not found, continue to next attribute in list */
 	      continue;
 	    } /* end if */
-	  }else{ /* Variable name does not contain '.' so variable presence is required */
+	  }else{ /* Variable name does not contain opt_chr so variable presence is required */
 	    rcd=nco_inq_varid(nc_id,var_nm,&var_id);
 	  } /* end if */
-	  if(rcd == NC_NOERR){
-	    if(att_rnm_lst[idx].old_nm[0] == '.'){
-	      /* Preceding '.' means attribute need not be present */
-	      rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm+1,&att_rnm_lst[idx].id);	  
-	      if(rcd == NC_NOERR){
-		(void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm);
-		nbr_rnm++;
-		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm,(var_nm[0] == '.' ? var_nm+1 : var_nm));
-	      }else{
-		(void)fprintf(stderr,"%s: WARNING Attribute \'%s\' not present in variable \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm+1,(var_nm[0] == '.' ? var_nm+1 : var_nm));
-	      } /* endelse */
-	    }else{ 
-	      rcd=nco_inq_attid(nc_id,var_id,att_rnm_lst[idx].old_nm,&att_rnm_lst[idx].id);
-	      if(rcd == NC_NOERR){
-		(void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
-		nbr_rnm++;
-		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm,(var_nm[0] == '.' ? var_nm+1 : var_nm));
-	      } /* endif attribute is present */
-	    } /* endelse attribute must be present */  
-	  }else{
-	    (void)fprintf(stderr,"%s: WARNING variable \'%s\' not present in %s\n",prg_nm,var_nm,fl_in);
-	  } /* endelse variable is present */
-	} /* end if renaming single variable */
-      }else{ /* ...or rename attribute for all variables... */
-	for(var_id=-1;var_id<nbr_var_fl;var_id++){ /* Start loop at -1 for global attributes */
-	  if(att_rnm_lst[idx].old_nm[0] == '.'){
-	    /* Rename attribute if variable contains attribute else do nothing */
-	    rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm+1,&att_rnm_lst[idx].id);
+	} /* !IS_GLB_GRP_ATT */
+	if(rcd == NC_NOERR){
+	  if(att_rnm_lst[idx].old_nm[0] == opt_chr){
+	    /* Preceding opt_chr means attribute need not be present */
+	    rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm+1L,&att_rnm_lst[idx].id);	  
 	    if(rcd == NC_NOERR){
-	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm);
+	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm+1L,att_rnm_lst[idx].new_nm);
+	      nbr_rnm++;
+	      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm+1L,att_rnm_lst[idx].new_nm,(var_nm[0] == opt_chr ? var_nm+1L : var_nm));
+	    }else{
+	      (void)fprintf(stderr,"%s: WARNING Attribute \'%s\' not present in variable \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm+1L,(var_nm[0] == opt_chr ? var_nm+1L : var_nm));
+	    } /* endelse */
+	  }else{ 
+	    /* Attribute must be present */
+	    rcd=nco_inq_attid(nc_id,var_id,att_rnm_lst[idx].old_nm,&att_rnm_lst[idx].id);
+	    if(rcd == NC_NOERR){
+	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
+	      nbr_rnm++;
+	      if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm,(var_nm[0] == opt_chr ? var_nm+1L : var_nm));
+	    } /* endif attribute is present */
+	  } /* endelse attribute must be present */  
+	}else{ /* variable not present */
+	  (void)fprintf(stderr,"%s: WARNING variable \'%s\' not present in %s\n",prg_nm,var_nm,fl_in);
+	}  /* variable not present */
+	/* end if renaming single variable */
+      }else{ /* ...or rename attribute for all variables... */
+	for(var_id=NC_GLOBAL;var_id<nbr_var_fl;var_id++){ /* Start loop at NC_GLOBAL (-1) for global attributes */
+	  if(var_id > NC_GLOBAL) (void)nco_inq_varname(nc_id,var_id,var_nm);
+	  if(att_rnm_lst[idx].old_nm[0] == opt_chr){
+	    /* Rename attribute if variable contains attribute else do nothing */
+	    rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm+1L,&att_rnm_lst[idx].id);
+	    if(rcd == NC_NOERR){
+	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm+1L,att_rnm_lst[idx].new_nm);
 	      nbr_rnm++;
 	      /* Inform user which variable had attribute renamed */
-	      if(var_id > -1){
-		char var_nm[NC_MAX_NAME];
-		
-		(void)nco_inq_varname(nc_id,var_id,var_nm);
-		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm,var_nm);
+	      if(var_id > NC_GLOBAL){
+		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm+1L,att_rnm_lst[idx].new_nm,var_nm);
 	      }else{
-		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed global attribute \'%s\' to \'%s\'\n",att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm);
+		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed global or group attribute \'%s\' to \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm+1L,att_rnm_lst[idx].new_nm);
 	      } /* end else */
 	    }else{ /* end if attribute was found */
 	      /* Reset error code */
 	      rcd=NC_NOERR; 
 	    } /* end else */
-	  }else{
+	  }else{ /* !opt_chr */
 	    /* Rename attribute or die trying */
 	    rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm,&att_rnm_lst[idx].id);
 	    if(rcd == NC_NOERR){
 	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
 	      nbr_rnm++;
 	      /* Inform user which variable had attribute renamed */
-	      if(var_id > -1){
-		char var_nm[NC_MAX_NAME];
-		
-		(void)nco_inq_varname(nc_id,var_id,var_nm);
-		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm,var_nm);
+	      if(var_id > NC_GLOBAL){
+		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm,var_nm);
 	      }else{
-		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"Renamed global attribute \'%s\' to \'%s\'\n",att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
+		IS_GLB_GRP_ATT=True; /* [flg] Attribute is Global or Group attribute */
+		if(IS_GLB_GRP_ATT) (void)fprintf(stderr,"%s: INFO found and renamed global or group attribute \'%s\' so not requiring its presence in every variable.\n",prg_nm_get(),att_rnm_lst[idx].old_nm); 
+		if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: Renamed global or group attribute \'%s\' to \'%s\'\n",prg_nm,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
 	      } /* end else */
-	    } /* end if */
-	  } /* end else */
+	    }else{ /* !NC_NOERR */
+	      /* Reset error code or print informative message and die */
+	      if(IS_GLB_GRP_ATT){
+		/* Forgive omission of period for global/group attributes. Users aren't perfect :) */
+		rcd=NC_NOERR;
+	      }else{ /* !IS_GLB_GRP_ATT */
+		(void)fprintf(stderr,"%s: ERROR User specified that presence of attribute \'%s\' is required. However, the %s%s does not contain it. HINT: If attribute presence is intended to be optional, then prefix attribute name with the period character \'%c\', e.g., %catt_nm. With this syntax %s would succeed even if no variables or groups contained the attribute. If attribute is intended to be renamed only in a specific variable, then prepend the variable name plus an at-sign \'%c\' to the attribute name, e.g., var_nm%catt_nm. If attribute presence is required only for global or group attributes, then prefix attribute name with \"global\" and an at-sign, e.g., global%catt_nm.\n",prg_nm_get(),att_rnm_lst[idx].old_nm,(var_id > NC_GLOBAL) ? "variable " : "root group",(var_id > NC_GLOBAL) ? var_nm : "",opt_chr,opt_chr,prg_nm_get(),dlm_chr,dlm_chr,dlm_chr);
+		/* Exit now rather than completing variable loop and printing lengthy error message above each iteration */
+		nco_err_exit(rcd,"main");
+	      } /* !IS_GLB_GRP_ATT */
+	    } /* !NC_NOERR */
+	  } /* !opt_chr */
 	} /* end loop over var_id */
       } /* end if renaming attribute for all variables */
       /* See to it that any mandatory renaming was performed, else abort */
       if(nbr_rnm == 0){
-	if(att_rnm_lst[idx].old_nm[0] == '.'){
-	  (void)fprintf(stderr,"%s: WARNING Attribute \'%s\' not renamed because not found in searched variable(s)\n",prg_nm,att_rnm_lst[idx].old_nm+1);
+	if(att_rnm_lst[idx].old_nm[0] == opt_chr){
+	  (void)fprintf(stderr,"%s: WARNING Attribute \'%s\' not renamed because not found in searched variable(s)\n",prg_nm,att_rnm_lst[idx].old_nm+1L);
 	}else{
 	  (void)fprintf(stdout,"%s: ERROR Attribute \'%s\' not present in %s, aborting.\n",prg_nm,att_rnm_lst[idx].old_nm,fl_in);
 	  nco_exit(EXIT_FAILURE);
 	} /* end else */
-      } /* end if */
-      
+      } /* nbr_rnm */
     } /* end loop over attributes to rename */
-     
   } /* end if renaming attributes */
   
   /* Catenate timestamped command line to "history" global attribute */
@@ -473,23 +516,25 @@ main(int argc,char **argv)
     (void)nco_enddef(nc_id);
   }else{
     (void)nco__enddef(nc_id,hdr_pad);
-    if(dbg_lvl >= nco_dbg_scl) (void)fprintf(stderr,"%s: INFO Padding header with %lu extra bytes \n",prg_nm_get(),(unsigned long)hdr_pad);
+    if(dbg_lvl >= nco_dbg_scl) (void)fprintf(stderr,"%s: INFO Padding header with %lu extra bytes\n",prg_nm_get(),(unsigned long)hdr_pad);
   } /* hdr_pad */
 
   /* Close the open netCDF file */
   nco_close(nc_id);
   
   /* Remove local copy of file */
-  if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
+  if(FL_RTR_RMT_LCN && RM_RMT_FL_PST_PRC) (void)nco_fl_rm(fl_in);
 
   /* Clean memory unless dirty memory allowed */
   if(flg_cln){
     /* ncrename-specific memory */
     for(idx=0;idx<nbr_att_rnm;idx++) att_rnm_arg[idx]=(char *)nco_free(att_rnm_arg[idx]);
     for(idx=0;idx<nbr_dmn_rnm;idx++) dmn_rnm_arg[idx]=(char *)nco_free(dmn_rnm_arg[idx]);
+    for(idx=0;idx<nbr_grp_rnm;idx++) grp_rnm_arg[idx]=(char *)nco_free(grp_rnm_arg[idx]);
     for(idx=0;idx<nbr_var_rnm;idx++) var_rnm_arg[idx]=(char *)nco_free(var_rnm_arg[idx]);
     if(nbr_att_rnm > 0) att_rnm_lst=(rnm_sct *)nco_free(att_rnm_lst);
     if(nbr_dmn_rnm > 0) dmn_rnm_lst=(rnm_sct *)nco_free(dmn_rnm_lst);
+    if(nbr_grp_rnm > 0) grp_rnm_lst=(rnm_sct *)nco_free(grp_rnm_lst);
     if(nbr_var_rnm > 0) var_rnm_lst=(rnm_sct *)nco_free(var_rnm_lst);
     
     /* NCO-generic clean-up */
@@ -513,4 +558,3 @@ main(int argc,char **argv)
   nco_exit_gracefully();
   return EXIT_SUCCESS;
 } /* end main() */
-
